# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4Color.Globals as Globals
import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.ValuePolicy
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import ResourceFiles as ResourceFiles
import ValuePolicy
import Utils.WeakMethod as WeakMethod
from QT4Color.ColorDropWidget import ColorDropWidget as ColorDropWidget
from QT4Color.ColorPolicy import DoesColorPolicyEnableFilmlook as DoesColorPolicyEnableFilmlook, DoesColorPolicyEnableNoFilmlookColorSpace as DoesColorPolicyEnableNoFilmlookColorSpace, DoesColorPolicyHaveAlpha as DoesColorPolicyHaveAlpha, GetColorPolicyChildren as GetColorPolicyChildren, GetColorPolicyRGBA as GetColorPolicyRGBA, GetDefaultColorComponentTab as GetDefaultColorComponentTab, SetColorPolicyRGBA as SetColorPolicyRGBA
from QT4Color.ColorTextWidget import ColorTextWidget as ColorTextWidget
from QT4Color.DropType import BuildDragDataFromInfo as BuildDragDataFromInfo, GetColorInfoFromDragObject as GetColorInfoFromDragObject, GetColorPolicyDragDict as GetColorPolicyDragDict
from QT4Color.Gradient1D import LinearGradientWidget as LinearGradientWidget
from QT4FormWidgets.GroupFormWidget import GroupFormWidget
from ResourceFiles.IconManager import ResourceManager as ResourceManager
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ColorComponentTabGroup(PyQt5.QtWidgets.QTabWidget):
    valueChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, policy, factory, enableFilmlook: Incomplete | None = ..., enableNoFilmlookColorSpace: Incomplete | None = ..., readOnly: bool = ..., rgbaTab: bool = ..., hslTab: bool = ..., hsvTab: bool = ..., tmlTab: bool = ..., tabToSelect: str = ...) -> None: ...
    def _ColorComponentTabGroup__colorComponentValuePolicy_CB(self, incomingPolicy, rawComponentValue, final): ...
    def _ColorComponentTabGroup__gradWidgetValueChangedCB(self, color, final): ...
    def _ColorComponentTabGroup__handleCurrentChanged(self, index): ...
    def eventFilter(self, obj, event): ...
    def getComponentFormWidgets(self): ...
    def setColor(self, color): ...
    def setColorPolicy(self, colorPolicy: ValuePolicy.AbstractValuePolicy): ...
    def setFilmlookEnabled(self, enabled): ...
    def setNoFilmlookColorSpaceEnabled(self, enabled): ...
    def setReadOnly(self, readOnly): ...
    def updateComponentReadOnlyStates(self): ...

class ColorFormWidget(GroupFormWidget):
    _ColorFormWidget__WARNING_PIXMAP_NAME: ClassVar[str] = ...
    def __init__(self, parent, policy, factory) -> None: ...
    def _ColorFormWidget__addUpdateThrottlerCallbackToChildPolicies(self): ...
    def _ColorFormWidget__colorComponent_valueChanged_CB(self, color, final): ...
    def _ColorFormWidget__colorDropWidget_dropEvent_CB(self, dropEvent): ...
    def _ColorFormWidget__colorTextWidget_valueChanged_CB(self, color): ...
    def _ColorFormWidget__update(self): ...
    def _buildAdditionalControlWidgets(self, layout): ...
    def _buildControlWidget(self, layout): ...
    def _buildInfoWidget(self): ...
    def _buildLabel(self, labelText, pos: int = ...): ...
    def _createChildWidget(self, policy, parentWidget, factory, index): ...
    def _freeze(self): ...
    def _lockChanged(self, state): ...
    def _participatesInLabelAlignment(self): ...
    def _popdownCreated(self, popdown): ...
    def _thaw(self): ...
    def execColorPicker(self): ...
    def getColorComponentTabGroup(self): ...
    def getDragDict(self, policy): ...
    def isDropOnlyMode(self): ...
    def isMiniMode(self): ...
    def setColor_RGBA(self, color, final: bool = ...): ...
    def showPopdown(self, value): ...
    def valueChangedEvent(self, event): ...

class ThinValuePolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    def __init__(self, name, hints) -> None: ...
    def _setInternalValue(self, value, final: bool = ...): ...
    def getName(self): ...
    def getType(self): ...
    def getValue(self): ...
    def getWidgetHints(self): ...
    def setCallback(self, callback): ...
    def setValue(self, value, final: bool = ...): ...

class UpdateThrottler(PyQt5.QtCore.QObject):
    updateSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent: Incomplete | None = ...) -> None: ...
    def _UpdateThrottler__update(self): ...
    def update(self, *args): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import QT4Widgets as QT4Widgets
import QT4Widgets.FilterablePopupButton
from typing import Set, Tuple

class SetVersionFilterablePopup(QT4Widgets.FilterablePopupButton.FilterablePopup):
    def __init__(self, *args) -> None: ...
    def popupWithAsset(self, assetId, pos): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import MuteAndSoloItemDelegates as MuteAndSoloItemDelegates
from UI4.Widgets.SceneGraphView.ItemDelegates.BaseItemDelegate import BaseItemDelegate as BaseItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.ColorItemDelegate import ColorItemDelegate as ColorItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.LightLinkItemDelegate import LightLinkItemDelegate as LightLinkItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.MuteAndSoloItemDelegates import MuteItemDelegate as MuteItemDelegate, SoloItemDelegate as SoloItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.NameItemDelegate import NameItemDelegate as NameItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.NumberItemDelegate import NumberItemDelegate as NumberItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.ParameterItemDelegate import ParameterItemDelegate as ParameterItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.ShaderItemDelegate import ShaderItemDelegate as ShaderItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.StateItemDelegate import StateItemDelegate as StateItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.WorkingSetItemDelegate import WorkingSetItemDelegate as WorkingSetItemDelegate
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
from QT4Widgets.FilterFieldWidget import FilterFieldWidget as FilterFieldWidget
from QT4Widgets.TreeWidgetUtil import UpdateSuppressor as UpdateSuppressor
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class BrowserColumn(PyQt5.QtWidgets.QFrame):
    itemDoubleClicked: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    itemSelectionChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent) -> None: ...
    def _BrowserColumn__doubleClick(self, index): ...
    def _BrowserColumn__treeMousePress(self, event): ...
    def _BrowserColumn__updateSelectNoneState(self): ...
    def filterItem(self, item, text, filterType): ...
    def getFilterField(self): ...
    def getFooter(self): ...
    def getHeader(self): ...
    def getSelectedItem(self): ...
    def getStatusLabel(self): ...
    def getTree(self): ...
    def selectItemByName(self, name): ...
    def singleVisibleItem(self): ...
    def updateFilter(self, text: Incomplete | None = ..., filterType: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import GeoAPI as GeoAPI
import QT4FormWidgets.InputWidgets as InputWidgets
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import UI4.KatanaPrefs.PrefNames as PrefNames
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.SortablePanel
from QT4FormWidgets.WidgetFactory import WidgetFactory
from UI4.Widgets.ComboBoxNoWheel import ComboBoxNoWheel as ComboBoxNoWheel
from UI4.Widgets.SceneGraphView.SceneGraphViewColumn import SceneGraphColumn as SceneGraphColumn
from UI4.Widgets.SortablePanel import SortablePanelBase as SortablePanelBase, SortablePanelFormWidget as SortablePanelFormWidget
from typing import Set, Tuple

class ScenegraphColumnDescriptionWidget(UI4.Widgets.SortablePanel.SortablePanelFormWidget):
    class ColumnAction(PyQt5.QtWidgets.QAction):
        def __init__(self, scenegraphColumnWidget, newColumnArgs: None, *args) -> None: ...
        def _ColumnAction__activatedCallback(self): ...
    def __init__(self, parent: PyQt5.QtWidgets.QWidget, policy: QT4FormWidgets.AbstractValuePolicy, factory: WidgetFactory) -> None: ...
    def _ScenegraphColumnDescriptionWidget__buildBaseValueFromPanels(self) -> str: ...
    def _ScenegraphColumnDescriptionWidget__buildExtValueFromPanels(self) -> str: ...
    def _ScenegraphColumnDescriptionWidget__processGenericAssignAttrs(self, attr, menu: PyQt5.QtWidgets.QMenu, parentName: str = ...): ...
    def _ScenegraphColumnDescriptionWidget__updateModel(self): ...
    def buildAddMenu(self, menu: PyQt5.QtWidgets.QMenu): ...
    @classmethod
    def getColumnPreferenceKeys(cls) -> None: ...
    def panelDeleted(self, index: Unused): ...
    def panelValueChanged(self, index: Unused): ...
    def updateLabel(self): ...
    def updatePanels(self): ...

class TagEditorPanel(UI4.Widgets.SortablePanel.SortablePanelBase):
    def __init__(self, parent: PyQt5.QtWidgets.QWidget, tags: None) -> None: ...
    def _TagEditorPanel__changedCallback(self, *args): ...
    def getAttrLocality(self) -> str: ...
    def getBaseValue(self) -> None: ...
    def getDisplayType(self) -> str: ...
    def getExtValue(self) -> None: ...
    def getName(self) -> str: ...
    def getNoneAttrValue(self) -> str: ...
    def isVisible(self) -> str: ...
    def updateEnabledState(self): ...

def buildPreferences(baseValue: str, extValue: str) -> tuple: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Util.GLDrawingRoutines as GLDrawingRoutines
import NodegraphAPI
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import UI4 as UI4
from UI4.FormMaster.Editors.Layers.LayerWithTransform import LayerWithTransform as LayerWithTransform
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class WorldCursorCircleLayer(LayerWithTransform, QT4GLLayerStack.LayerStack.Layer):
    _CROSSHAIR_SIZE: ClassVar[int] = ...
    _CROSSHAIR_START: ClassVar[int] = ...
    _CROSSHAIR_THRESHOLD: ClassVar[int] = ...
    def __init__(self, node: NodegraphAPI.Node, radius: Incomplete | None = ..., **kwargs) -> None: ...
    def _drawCursor(self, cursorPos, radius, color): ...
    def getCursorPos(self): ...
    def getRadius(self): ...
    def paintGL(self): ...
    def setCursorPos(self, newCursorPos): ...
    def setRadius(self, r): ...
    def setVisible(self, visible): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.HintUtils as HintUtils
import QT4FormWidgets.PolicyMimeData as PolicyMimeData
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import Utils.WeakMethod as WeakMethod
import re
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class AbstractValuePolicy:
    SUBSCRIPT_EXPR: ClassVar[re.Pattern] = ...
    _QueuedChanged: ClassVar[dict] = ...
    def __init__(self, parent) -> None: ...
    def _AbstractValuePolicy__parentNameChanged(self, parentName): ...
    @staticmethod
    def _HandleQueuedChanges(): ...
    def _appendToContextMenu(self, menu: PyQt5.QtWidgets.QMenu): ...
    def _appendToStateMenu(self, menu: PyQt5.QtWidgets.QMenu) -> bool: ...
    def _appendToWrenchMenu(self, menu): ...
    def _childStateChange(self): ...
    def _childStateReset(self): ...
    def _closeButtonClicked(self): ...
    def _customizeWrenchButton(self, wrenchButton: PyQt5.QtWidgets.QPushButton): ...
    def _freeze(self): ...
    def _handleValuePolicyChange(self, event: ValuePolicyEvent): ...
    def _potentialFreezeChange(self, anyHandlers: Incomplete | None = ..., active: Incomplete | None = ..., frozenTop: Incomplete | None = ...): ...
    def _propagateNameChange(self, policy: ValuePolicy): ...
    def _propagateStateChange(self, visitedParents: set = ...): ...
    def _thaw(self): ...
    def addCallback(self, callback): ...
    def addWeakCallback(self, callback): ...
    def bakeCurve(self, keys): ...
    def canHaveCurve(self): ...
    def canHaveExpression(self): ...
    def checkMimeData(self, mimeData): ...
    def delCallback(self, callback): ...
    def delWeakCallback(self, callback): ...
    def findRelativePolicy(self, pathList, index: int = ...): ...
    def getArrayChild(self, index): ...
    def getArrayChildren(self): ...
    def getArraySize(self): ...
    def getAvailableStates(self): ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getCurve(self): ...
    def getCurveAutoKey(self): ...
    def getCurveKey(self): ...
    def getCurveXml(self): ...
    def getExpression(self): ...
    def getExpressionError(self): ...
    def getExpressionNamespace(self): ...
    def getExpressionReference(self): ...
    def getFullName(self): ...
    def getMimeData(self) -> PyQt5.QtCore.QMimeData: ...
    def getName(self): ...
    def getNumChildren(self): ...
    def getOpenStateKey(self): ...
    def getParent(self): ...
    def getSceneGraphLocationPath(self) -> str | None: ...
    def getStateAppearance(self, state): ...
    def getTooltip(self) -> str: ...
    def getType(self): ...
    def getValue(self): ...
    def getValueAsString(self): ...
    def getValueState(self): ...
    def getWidgetHints(self): ...
    def hasFloatingCurve(self): ...
    def isCloseButtonEnabled(self) -> bool: ...
    def isCurveEnabled(self): ...
    def isCurveViewed(self): ...
    def isExpressionEnabled(self): ...
    def isFrozen(self): ...
    def isGroupParameterBeingEdited(self): ...
    def isLocked(self): ...
    def isValid(self) -> bool: ...
    def loadCurveFromFile(self, path): ...
    def removeArrayChild(self, index): ...
    def reorderArrayChild(self, oldIndex, newIndex): ...
    def setArraySize(self, size): ...
    def setCurve(self, curve): ...
    def setCurveAutoKey(self, autoKey): ...
    def setCurveEnabled(self, state): ...
    def setCurveKey(self, state): ...
    def setCurveViewed(self, state): ...
    def setCurveXml(self, xml): ...
    def setExpression(self, expr): ...
    def setExpressionEnabled(self, flag): ...
    def setMimeData(self, mimeData, dropAction): ...
    def setParent(self, parent: AbstractValuePolicy | None): ...
    def setValue(self, value, final: bool = ...): ...
    def setValueFromString(self, value, final: bool = ...): ...
    def setValueState(self, state): ...
    def shouldDisplayClose(self) -> bool: ...
    def shouldDisplayState(self): ...
    def shouldDisplayWrench(self): ...
    def toggleValueState(self): ...
    def valueChanged(self, value: bool = ..., final: bool = ..., stateChanged: bool = ..., topologyChanged: bool = ..., nameChanged: bool = ...): ...
    def waitForReady(self): ...
    def writeCurveToFile(self, path): ...

class ValuePolicyEvent:
    def __init__(self, policy, value, final, stateChanged, topologyChanged, nameChanged) -> None: ...
    def _updateChanges(self, value, final, stateChanged, topologyChanged, nameChanged): ...
    def getPolicy(self): ...
    def hasNameChanged(self): ...
    def hasStateChanged(self): ...
    def hasTopologyChanged(self): ...
    def hasValueChanged(self): ...
    def isFinal(self): ...

class ValuePolicyProxy:
    def __init__(self, policy) -> None: ...
    def _ValuePolicyProxy__valueChangedEvent(self, event): ...
    def addCallback(self, callback): ...
    def addWeakCallback(self, callback): ...
    def delCallback(self, callback): ...
    def delWeakCallback(self, callback): ...
    def findRelativePolicy(self, pathList, index: int = ...): ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getFullName(self): ...
    def getMimeData(self): ...
    def getName(self): ...
    def getPolicy(self): ...
    def getTooltip(self) -> str: ...
    def getWidgetHints(self): ...
    def setName(self, overrideName): ...
    def setPolicy(self, policy): ...
    def setWidgetHints(self, overrideHints): ...
    def __getattr__(self, name): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFCurve as PyFCurve
import Utils as Utils
from typing import Set, Tuple

class FCurveSelectionSnapshot(PyFCurve.SelectionObserver):
    def __init__(self, container, selection) -> None: ...
    def _FCurveSelectionSnapshot__keyframeSelectTuple(self, keyframe): ...
    def deselected(self, object): ...
    def restoreSelection(self): ...
    def restoreSelectionConditional(self): ...
    def selected(self, object): ...
    def selectionCleared(self): ...
    def setIgnoreChanges(self, ignore): ...
    def snapshotSelection(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.EnableableParameterPolicy as EnableableParameterPolicy
import UI4.FormMaster.HintsDelegate as HintsDelegate
import UI4.FormMaster.ParameterPolicy as ParameterPolicy
from UI4.FormMaster.NodeHints.Common2D import GetCapsuleRgbaDict as GetCapsuleRgbaDict
from typing import Set, Tuple

_NodeHints: dict
_NodeName: str



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Tabs.DopeSheet.DopeSheetTree as DopeSheetTree
import NodegraphAPI as NodegraphAPI
import UI4.KatanaPrefs.PrefNames as PrefNames
import PyQt5.QtCore
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import UI4.Util.Events
import Utils as Utils
from PyUtilModule.VirtualKatana import ParameterKeyMimeData as ParameterKeyMimeData
from UI4.KatanaPrefs.KatanaPrefsObject import KatanaPrefs as KatanaPrefs
from UI4.Util.Events import EventProcessor as EventProcessor, EventProcessorHandler as EventProcessorHandler, LayerWorldDragEventProcessor as LayerWorldDragEventProcessor
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class BandSelectionEventProcessor(UI4.Util.Events.LayerWorldDragEventProcessor):
    def __init__(self, layerStack) -> None: ...
    def _BandSelectionEventProcessor__scrollLayerStack(self, dx, dy): ...
    def _BandSelectionEventProcessor__updateSelection(self, modifiers): ...
    def _update(self, worldStart, worldEnd, modifiers, initial: bool = ..., endChanged: bool = ..., modifiersChanged: bool = ..., final: bool = ...): ...
    def finish(self, cancelled: bool = ...): ...

class BandSelectionLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _BandSelectionLayer__updateRect(self): ...
    def getEnd(self): ...
    def getRectangle(self): ...
    def getStart(self): ...
    def paintGL(self): ...
    def setEnd(self, x1, y1): ...
    def setStart(self, x0, y0): ...

class CurrentTimeEventProcessor(UI4.Util.Events.LayerWorldDragEventProcessor):
    def _update(self, worldStart, worldEnd, modifiers, initial: bool = ..., endChanged: bool = ..., modifiersChanged: bool = ..., final: bool = ...): ...
    def finish(self, cancelled: bool = ...): ...

class KeyInteractionLayer(QT4GLLayerStack.LayerStack.Layer, UI4.Util.Events.EventProcessorHandler):
    def __init__(self, treeWidget, *args, **kwargs) -> None: ...
    def _KeyInteractionLayer__copySelectedKeys(self): ...
    def _KeyInteractionLayer__deleteSelectedKeys(self): ...
    def _KeyInteractionLayer__getSelection(self, rootItem: Incomplete | None = ..., paramRoot: Incomplete | None = ...): ...
    def _KeyInteractionLayer__getSelectionHierarchy(self): ...
    def _KeyInteractionLayer__getSelectionHierarchyR(self, rootItem, paramRoot): ...
    def _KeyInteractionLayer__getToolTipText(self, layerStack, event): ...
    def _KeyInteractionLayer__pasteKeys(self): ...
    def _KeyInteractionLayer__setKey(self): ...
    def _KeyInteractionLayer__setKeyOnAll(self): ...
    def _KeyInteractionLayer__setKeyR(self, entry, t, paramRoot): ...
    def _KeyInteractionLayer__toggleToolTips(self): ...
    def _KeyInteractionLayer__updateActions(self, layerStack): ...
    def _KeyInteractionLayer__updateGhostTime(self, layerStack, pos: Incomplete | None = ..., updateGL: bool = ...): ...
    def _processEvent(self, event): ...
    def _processEventUnconditional(self, event): ...
    def buildContextMenu(self, menu, contextMenuEvent): ...
    def contextMenuFinished(self): ...
    def processEvent(self, event): ...
    def setLayerStack(self, *args): ...

class KeyMoveEventProcessor(UI4.Util.Events.LayerWorldDragEventProcessor):
    _KeyMoveEventProcessor__COPY_KEY: ClassVar[PyQt5.QtCore.Qt.Key] = ...
    _KeyMoveEventProcessor__COPY_MODIFIER: ClassVar[PyQt5.QtCore.Qt.KeyboardModifier] = ...
    _KeyMoveEventProcessor__OVERWRITE_KEY: ClassVar[PyQt5.QtCore.Qt.Key] = ...
    _KeyMoveEventProcessor__OVERWRITE_MODIFIER: ClassVar[PyQt5.QtCore.Qt.KeyboardModifier] = ...
    def __init__(self, layerStack, treeWidget) -> None: ...
    def _KeyMoveEventProcessor__checkAllowed(self): ...
    def _KeyMoveEventProcessor__scrollLayerStack(self, dx, dy): ...
    def _KeyMoveEventProcessor__showCopyCursor(self, showAsCopy): ...
    def _KeyMoveEventProcessor__showMove(self): ...
    def _update(self, worldStart, worldEnd, modifiers, initial: bool = ..., endChanged: bool = ..., modifiersChanged: bool = ..., final: bool = ...): ...
    def finish(self, cancelled: bool = ...): ...

class PasteTarget:
    def __init__(self, treeWidget) -> None: ...
    def _PasteTarget__getMimeData(self): ...
    def _PasteTarget__getParametersFromEntry(self, entry): ...
    def _PasteTarget__getTargetParameters(self, mimeData): ...
    def _PasteTarget__unwrapPotentials(self, potentialsHierarchy): ...
    def clear(self): ...
    def getPasteDescription(self): ...
    def getPotentials(self): ...
    def getTargetEntry(self): ...
    def getTargetRootEntry(self): ...
    def getTargetTime(self): ...
    def isTargetTimeSet(self): ...
    def isValid(self): ...
    def paste(self): ...
    def setTargetEntry(self, entry): ...
    def setTargetTime(self, time): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI.BypassParameterManager as BypassParameterManager
import DrawingModule as DrawingModule
import UI4.Util.IconManager as IconManager
import KatanaResources as KatanaResources
import UI4.Util.NodeDialogs as NodeDialogs
import UI4.FormMaster.NodeMimeData as NodeMimeData
import NodegraphAPI as NodegraphAPI
import PyQt5.QtGui
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import Utils as Utils
from NodegraphAPI.LiveGroup import LiveGroupMixin as LiveGroupMixin
from UI4.App.KeyboardShortcutManager.KeyboardShortcutManagerMixin import KeyboardShortcutManagerMixin as KeyboardShortcutManagerMixin, KeyboardShortcutManagerMixinMetaclass as KeyboardShortcutManagerMixinMetaclass
from UI4.Widgets.NodePopupMenu import NodePopupMenu as NodePopupMenu
from UI4.Widgets.ToolbarButton import ToolbarButton as ToolbarButton
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

__liveGroupPixmapNames: dict
__nodePixmaps: dict
eventType: str

class NodeButton(ToolbarButton, KeyboardShortcutManagerMixin):
    BypassPixmap: ClassVar[None] = ...
    EditPixmap: ClassVar[None] = ...
    MinimumIndicatorSize: ClassVar[int] = ...
    Padding: ClassVar[int] = ...
    ViewPixmap: ClassVar[None] = ...
    _NodeButton__HasRegisteredKeyboardShortcuts: ClassVar[bool] = ...
    _NodeButtons: ClassVar[list] = ...
    def __init__(self, node: NodegraphAPI.Node | None, defaultTooltip: str = ..., parent: Incomplete | None = ...) -> None: ...
    def _NodeButton__buildIcon(self, hover: bool = ...): ...
    def _NodeButton__getTooltip(self, node: NodegraphAPI.Node | None, defaultTooltip: str) -> str: ...
    def _NodeButton__setEditFlag(self): ...
    def _NodeButton__setViewFlag(self): ...
    def _NodeButton__startNodeDrag(self): ...
    def _NodeButton__toggleBypassedState(self): ...
    def _NodeButton__toggleEditFlag(self): ...
    def _NodeButton__toggleViewFlag(self): ...
    def _updateIcon(self): ...
    def _updateStyleSheet(self, forceHoverColor: bool = ...): ...
    def changeEvent(self, event): ...
    def customEvent(self, event): ...
    def editNodeName(self): ...
    def getNode(self) -> NodegraphAPI.Node | None: ...
    def mousePressEvent(self, event: PyQt5.QtGui.QMouseEvent): ...
    @classmethod
    def registerKeyboardShortcuts(cls): ...
    def setBold(self, bold: bool): ...
    def setNode(self, node: NodegraphAPI.Node): ...
    def showContextMenu(self, pos): ...
    def updateAppearance(self): ...

def _GetNodePixmap(node: NodegraphAPI.Node) -> PyQt5.QtGui.QPixmap: ...
def _InitNodePixmaps(): ...
def __on_node_updated(args: list[tuple[str, object, dict]]): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Callbacks as Callbacks
import ConfigurationAPI_cmodule as Configuration
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import GeoAPI.Util.LookFileUtil as LegacyLookFileUtil
import LookFileBakeAPI as LookFileBakeAPI
import LookFileBakeAPI.LookFileBaker
import Nodes3DAPI.Node3D as Node3D
import NodegraphAPI as NodegraphAPI
import PyFnGeolib
import PyFnGeolibProducers
import typing
from _typeshed import Incomplete
from typing import Set, Tuple

class PostLookFileBakeHandler(_PrePostLookFileBakeCallbackForwarder):
    def __init__(self, node: typing.Optional[NodegraphAPI.Node] = ...) -> None: ...

class PreLookFileBakeHandler(_PrePostLookFileBakeCallbackForwarder):
    def __init__(self, node: typing.Optional[NodegraphAPI.Node] = ...) -> None: ...

class _PrePostLookFileBakeCallbackForwarder(LookFileBakeAPI.LookFileBaker.BakePrePostHandlerBase):
    callback: Incomplete
    node: Incomplete
    def __init__(self, node: typing.Optional[NodegraphAPI.Node] = ...) -> None: ...
    def notify(self, assetId: str, rootLocationProducers: dict[str, PyFnGeolibProducers.GeometryProducer], progressCallback: typing.Callable | None, abortCallback: typing.Callable | None): ...

def BakingContext(): ...
def GetBakeState() -> bool: ...
def GetGlobalBakeState() -> bool: ...
def GetLocationIntervalEvictor(node: typing.Optional[NodegraphAPI.Node] = ..., intervalParameterName: str = ..., configVarName: str = ...) -> LocationIntervalEvictor | None: ...
def GetLookFileBakeOp(node: NodegraphAPI.Node, inputPortName: str, graphState: typing.Optional[Nodegraph], txn: PyFnGeolib.GeolibRuntimeTransaction | None) -> PyFnGeolib.GeolibRuntimeOp | None: ...
def GetLookFileBakeOps(node: NodegraphAPI.Node, inputPortNames: Sequence[str], graphState: typing.Optional[Nodegraph], txn: Incomplete | None = ...) -> list[PyFnGeolib.GeolibRuntimeOp | None]: ...
def PostLookFileBake(node: NodegraphAPI.Node | None, assetId: str, rootLocationProducers: dict[str, PyFnGeolibProducers.GeometryProducer], progressCallback: typing.Callable | None, abortCallback: typing.Callable | None): ...
def PreLookFileBake(node: NodegraphAPI.Node | None, assetId: str, rootLocationProducers: dict[str, PyFnGeolibProducers.GeometryProducer], progressCallback: typing.Callable | None, abortCallback: typing.Callable | None): ...
def ValidateGraphState(graphStateOrTime: typing.Optional[Nodegraph]) -> NodegraphAPI.GraphState: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4GLLayerStack.LayerStack
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
from QT4GLLayerStack.LayerStack import Layer as Layer
from typing import Set, Tuple

class RectangleLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _RectangleLayer__fillBackground(self): ...
    def _RectangleLayer__findDragEdges(self, localX, localY): ...
    def _RectangleLayer__paintDimBackground(self): ...
    def _RectangleLayer__pointInCenter(self, localX, localY): ...
    def _RectangleLayer__pointInRect(self, localX, localY, includeGravity: bool = ...): ...
    def _RectangleLayer__pointOnAxis(self, localX, localY): ...
    def beginEdgeDrag(self, xEdge, yEdge): ...
    def boxColor(self): ...
    def center(self): ...
    def clamped(self): ...
    def defaultProcessEvent(self, event): ...
    def dimBackground(self): ...
    def displayRectangle(self): ...
    def drawBox(self): ...
    def edgeDragProcessEvent(self, event): ...
    def endEdgeDrag(self): ...
    def intermediate(self): ...
    def originLock(self): ...
    def outlineWidth(self): ...
    def paintGL(self): ...
    def processEvent(self, event): ...
    def rectangle(self): ...
    def rectangleDragProcessEvent(self, event): ...
    def setBoxColor(self, color): ...
    def setCenter(self, center): ...
    def setClamped(self, v): ...
    def setDimBackground(self, flag): ...
    def setDrawBox(self, drawFlag): ...
    def setOriginLock(self, v): ...
    def setOutlineWidth(self, width): ...
    def setRectangle(self, rect): ...
    def setSize(self, size): ...
    def size(self): ...
    def windowRectangle(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
import PyFnAttribute as FnAttribute
import GeoAPI as GeoAPI
import NodegraphAPI as NodegraphAPI
import PyOpenColorIO as OCIO
import PluginSystemAPI as PluginSystemAPI
import PyFnGeolibProducers
import RenderingAPI.RenderOutputs as RenderOutputs
import RenderingAPI as RenderingAPI
import re
import typing
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class GetRendererPluginDirError(RuntimeError): ...

class PluginBase:
    _PluginBase__misc_resolution_re: ClassVar[re.Pattern] = ...
    def _convertResolution(self, resolution): ...

def GetAllRenderMethods(includeInternal: bool = ...) -> list[dict]: ...
def GetBatchRenderMethodForRenderer(rendererName): ...
def GetDefaultRendererPluginName() -> str | None: ...
def GetInfoPlugin(rendererName): ...
def GetInfoPluginName(rendererName): ...
def GetInfoPluginNameForShaderType(shaderType): ...
def GetOutputChannelAttr(rendererName, dynamicOnly: bool = ...): ...
def GetOutputChannelDynamicParameters(rendererName): ...
def GetOutputChannelGlobalStatementsAttr(rendererName): ...
def GetOutputChannelParameters(rendererName): ...
def GetProcessedParams(paramList, hintDict: dict = ...): ...
def GetRenderMethodByName(methodName, rendererName: Incomplete | None = ...): ...
def GetRenderMethodForRendererAndMethodName(rendererName, methodName, includeBatch: bool = ...): ...
def GetRenderMethodsForRenderer(rendererName): ...
def GetRendererPluginDir(name): ...
def GetRendererPluginDisplayColor(rendererName: str): ...
def GetRendererPluginDisplayName(rendererName: str) -> str: ...
def GetRendererPluginNameForShaderType(shaderType): ...
def GetRendererPluginNames(includeViewer: bool = ..., includeInternal: bool = ...) -> list[str]: ...
def GetRendererSettingsAttr(outputType, producer: typing.Optional[PyFnGeolibProducers.GeometryProducer] = ..., producerBaseName: Incomplete | None = ...): ...
def GetRendererShaderTypeAttrPrefixes(rendererName: Incomplete | None = ...): ...
def GetRendererShaderTypeParamsAttr(rendererName: Incomplete | None = ...): ...
def GetRendererShaderTypeShaderAttr(rendererName: Incomplete | None = ...): ...
def GetRendererShaderTypes(renderer: Incomplete | None = ...): ...
def SetRendererPluginDisplayColor(rendererName: str, color): ...
def SetRendererPluginDisplayName(rendererName: str, displayName: str): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.Editors.Layers.Transform2D as Transform2D
import UI4.FormMaster.Editors.Layers.Transform2D
import Utils as Utils
from typing import Set, Tuple

class RepositionManipLayer(UI4.FormMaster.Editors.Layers.Transform2D.Transform2DManipLayer):
    def __init__(self, valuePolicy, *args, **kwargs) -> None: ...
    def getTranslate(self): ...
    def hasRotate(self): ...
    def hasScale(self, horizontal): ...
    def hasTranslate(self, horizontal): ...
    def setTranslate(self, value, final): ...
    def translateIsExpressioned(self, horizontal): ...
    def translateIsLocked(self, horizontal): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import CacheManager as CacheManager
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import PyFnGeolibServices as FnGeolibServices
import NodegraphAPI as NodegraphAPI
import NodegraphAPI_cmodule as NodegraphAPI_cmodule
import Nodes3DAPI
import PyFnGeolib
import Nodes3DAPI.UpdateModes as UpdateModes
import Utils as Utils
import _weakrefset
import typing
import weakref
from Nodes3DAPI.PortOpClient import PortOpClient as PortOpClient, g_portOpClientRegistry as g_portOpClientRegistry
from Nodes3DAPI.TerminalOpDelegates.TerminalOpDelegate import TerminalOpDelegate
from PyFnGeolib import _Geolib as _Geolib, _Runtime as _Runtime, _gDefaultSourceOp as _gDefaultSourceOp
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class NodeGeolib3:
    class OpChainInterface:
        FAIL: ClassVar[int] = ...
        NO_OP: ClassVar[int] = ...
        SKIP: ClassVar[int] = ...
        def __init__(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, entries, inputRequests) -> None: ...
        def addInputRequest(self, inputPortName, graphState: NodegraphAPI.GraphState, invalidInputBehavior: int = ...): ...
        def appendOp(self, opType, opArgs): ...
        def getGraphState(self): ...
        def getMinRequiredInputs(self): ...
        def getPort(self): ...
        def setExplicitInputRequestsEnabled(self, state): ...
        def setMinRequiredInputs(self, value): ...

    class OpRecord:
        def __init__(self, transaction, nodeName, nodeType) -> None: ...

    class ParamDependencyTracker:
        def __init__(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState) -> None: ...
        def addDependency(self, node: NodegraphAPI.Node): ...
        def __enter__(self): ...
        def __exit__(self, type, value, traceback): ...
    _NodeGeolib3__instances: ClassVar[_weakrefset.WeakSet] = ...
    _NodeGeolib3__opIdToNodeDict: ClassVar[weakref.WeakValueDictionary] = ...
    def __init__(self) -> None: ...
    def _NodeGeolib3__handleInvalidInput(self, transaction, graphState: NodegraphAPI.GraphState, inputOps, inputPortName, invalidInputBehavior, message): ...
    def _areInputsAndParametersDirty(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, visitedState: set, reset: bool = ...) -> Tuple[bool, bool]: ...
    def _areInputsDirty(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, visitedState, reset: bool = ...): ...
    def _areParamsDirty(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, reset: bool = ...): ...
    def _areParentsDirty(self, graphState: NodegraphAPI.GraphState, reset: bool = ...) -> bool: ...
    def _createOrGetOp(self, transaction, key, graphState: NodegraphAPI.GraphState): ...
    def _flushAll(self): ...
    def _getErrorOp(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, transaction: PyFnGeolib.GeolibRuntimeTransaction, message: str) -> PyFnGeolib.GeolibRuntimeOp: ...
    def _getOp(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, visitedState: set, transaction: PyFnGeolib.GeolibRuntimeTransaction) -> PyFnGeolib.GeolibRuntimeOp: ...
    def _getSafeOp(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, visitedState, transaction): ...
    def _getSafeOpChain(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, visitedState, transaction): ...
    def _registerOpChain(self, opChain): ...
    def _setParamsDirty(self): ...
    @staticmethod
    def _unregisterOpChain(opChain): ...
    @staticmethod
    def flushAll(): ...
    @staticmethod
    def getNodeForOpId(opId: int) -> Nodes3DAPI.Node3D | None: ...
    def invalidateOps(self): ...

class _NodeTraversal(NodeTraversal):
    @classmethod
    def __init__(cls, graphState: NodegraphAPI.GraphState, nodeTraversalEndpoints) -> None: ...

class _PortAndGraphState(tuple):
    _field_defaults: ClassVar[dict] = ...
    _fields: ClassVar[tuple] = ...
    _fields_defaults: ClassVar[dict] = ...
    def __init__(self, _cls, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState) -> None: ...
    def _asdict(self): ...
    @classmethod
    def _make(cls, iterable): ...
    def _replace(self, _self, **kwds): ...
    def __getnewargs__(self): ...
    @property
    def graphState(self): ...
    @property
    def port(self): ...

def ActivatePort(port: NodegraphAPI.Port, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., potentialOpChangeCallback: typing.Optional[typing.Callable] = ..., getOpFunction: Incomplete | None = ..., applyTerminalOpDelegatesForCallback: bool = ...): ...
def ApplyTerminalOpDelegates(op: PyFnGeolib.GeolibRuntimeOp, txn: PyFnGeolib.GeolibRuntimeTransaction, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState) -> PyFnGeolib.GeolibRuntimeOp: ...
def CommitChanges() -> int: ...
def DeactivatePort(port: NodegraphAPI.Port, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., potentialOpChangeCallback: typing.Optional[typing.Callable] = ...): ...
def GetDefaultSourceOp(): ...
def GetRuntime(): ...
def IsProcessing(): ...
def ManualUpdate() -> int: ...
def MarkPortOpClientDirty(portOpClient: PortOpClient): ...
def PrintOpDependencyInfo(): ...
def RegisterCommitIdCallback(commitIdCallback: typing.Callable): ...
def RegisterPortOpClient(portOpClient: PortOpClient): ...
def RegisterTerminalOpDelegate_geolib3(name: str, terminalOpDelegate: TerminalOpDelegate): ...
def UnregisterPortOpClient(portOpClient: PortOpClient): ...
def UpdatePortOpClients(portOpClients: Incomplete | None = ...) -> int: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI.Flavor as Flavor
import NodegraphAPI.NodeExtensions as NodeExtensions
import NodegraphAPI
import NodegraphAPI_cmodule as NodegraphAPI_cmodule
import NodegraphAPI.NodegraphGlobals as NodegraphGlobals
import Utils as Utils
from typing import Set, Tuple

class SuperTool(NodegraphAPI_cmodule.PythonGroupNode):
    def __init__(self) -> None: ...
    def getNodesCreatingSceneGraphLocations(self) -> list[NodegraphAPI.Node | None]: ...
    def hideNodegraphGroupControls(self): ...
    def isChildNodePersistent(self, node: NodegraphAPI.Node) -> bool: ...
    def isUserParameterEditingAllowed(self) -> bool: ...
    def polish(self): ...
    @classmethod
    def superToolRegistered(cls): ...

def RegisterSuperTool(name, superToolClass): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import PyOpenColorIO as OCIO
import PyFnGeolibProducers
import RenderingAPI as RenderingAPI
import typing
from _typeshed import Incomplete
from typing import Set, Tuple

def GetDefaultRendererSettingsAttr(outputType, producer: typing.Optional[PyFnGeolibProducers.GeometryProducer] = ..., producerBaseName: Incomplete | None = ...): ...
def GetRenderOutputs(client, includeMerge: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from _typeshed import Incomplete
from typing import Set, Tuple

class ExpandingLabel(PyQt5.QtWidgets.QTextEdit):
    def __init__(self, *args) -> None: ...
    def _rightAlign(self): ...
    def minimumSizeHint(self): ...
    def resizeEvent(self, resizeEvent): ...
    def setText(self, qstring, color: Incomplete | None = ...): ...
    def sizeHint(self): ...
    def sizePolicy(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import CatalogAPI as CatalogAPI
import PyUtilModule.EnvUtils as EnvUtils
import UI4.Util.ExternalTools as ExternalTools
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from typing import Set, Tuple

class RenderLogTab(BaseTab):
    def __init__(self, parent) -> None: ...
    def _RenderLogTab__actionMenuAboutToShow(self): ...
    def _RenderLogTab__catalog_itemPropertyUpdate_CB(self, eventType, eventID, item): ...
    def _RenderLogTab__catalog_rebuild_CB(self, args: list): ...
    def _RenderLogTab__clearContents_CB(self): ...
    def _RenderLogTab__numLinesChanged_CB(self): ...
    def _RenderLogTab__on_renderLogWidget_autoScrollingChanged(self, autoScrolling: bool): ...
    def _RenderLogTab__on_scrollLockToolButton_clicked(self): ...
    def _RenderLogTab__openInEditor_CB(self): ...
    def _RenderLogTab__updateScrollLockToolButton(self, autoScrolling: bool): ...
    def copySelectedTextActionCallback(self): ...
    def getCatalogItem(self): ...
    @staticmethod
    def registerKeyboardShortcuts(): ...
    def setCatalogItem(self, item): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
import PyFnGeolibServices as FnGeolibServices
import KatanaResources as KatanaResources
import PluginSystemAPI as PluginSystemAPI
import PyFnGeolibProducers as PyFnGeolibProducers
import PyFnGeolibProducers.RendererOutputUtil as RendererOutputUtil
from . import Lookfiles as Lookfiles, Transform as Transform, Util as Util
from GeoAPI.Transform import ProducerLocalTransform as ProducerLocalTransform, ProducerWorldBounds as ProducerWorldBounds, ProducerWorldTransform as ProducerWorldTransform, UnionBounds as UnionBounds
from PyFnGeolibProducers import GeometryProducer as GeometryProducer
from PyFnGeolibProducers.RendererOutputUtil import __GetCameraInfoUnwrapped as __GetCameraInfoUnwrapped, __GetLocalToScreenMatrixUnwrapped as __GetLocalToScreenMatrixUnwrapped, __GetProjectionMatrixUnwrapped as __GetProjectionMatrixUnwrapped
from typing import Set, Tuple

Geolib3BaseDir: str
secondaryPluginPaths: list
usingFlags: bool

def __GetCameraInfoWrapped(*args): ...
def __GetLocalToScreenMatrixWrapped(): ...
def __GetProjectionMatrixWrapped(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Nodes3DAPI.DynamicParameterUtil as DynamicParameterUtil
import PyFnAttribute as FnAttribute
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI_cmodule as Nodes3DAPI_cmodule
import PyXmlIO as PyXmlIO
import RenderingAPI.RenderPlugins as RenderPlugins
import RenderingAPI_cmodule.RendererInfo as RendererInfo
import RenderingAPI as RenderingAPI
import Naming as UniqueName
import Utils as Utils
from Nodes3DAPI.Node3D import Node3D as Node3D
from _typeshed import Incomplete
from typing import Set, Tuple

class OutputChannelDefine(Node3D):
    def __init__(self, rendererName) -> None: ...
    def AddGroup(self, typeName: Incomplete | None = ...): ...
    def DeleteGroup(self, index): ...
    def GetAvailableTypes(self): ...
    def ReorderGroup(self, index, newIndex): ...
    def _OutputChannelDefine__applyCustom(self, element): ...
    def _OutputChannelDefine__applyType(self, element, typeName, typeInfo): ...
    def _OutputChannelDefine__convertValueParamToAttr(self, type, valueParam, frameTime): ...
    def _OutputChannelDefine__getGroupNameAndAttrPrefix(self, frameTime): ...
    def _OutputChannelDefine__getNameValue(self, groupName): ...
    def _OutputChannelDefine__initRendererParameters(self, rendererName): ...
    def _OutputChannelDefine__rebuildDynamicParameters(self, dynamicParams, frameTime, universalAttr): ...
    def _getIncomingSceneOpAndLocation(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, transaction): ...
    def _getOpChain(self, interface): ...
    def _groupAttrToDict(self, groupAttr): ...
    def _updateParameters(self, groupAttr, force: bool = ..., defaultAttr: Incomplete | None = ...): ...
    def addParameterHints(self, attrName, inputDict): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI
import PyFnGeolib
from typing import Set, Tuple

class TerminalOpDelegate:
    def appendOp(self, op: PyFnGeolib.GeolibRuntimeOp, txn: PyFnGeolib.GeolibRuntimeTransaction, port: NodegraphAPI.Port | None, graphState: NodegraphAPI.GraphState) -> PyFnGeolib.GeolibRuntimeOp: ...
    def update(self) -> bool: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets as QT4FormWidgets
import UI4.FormMaster.Editors.SortableArray as SortableArray
import UI4.FormMaster.Editors.SortableArray
import Utils as Utils
from QT4FormWidgets.StringFormWidget import StringFormWidget
from typing import Set, Tuple

class AttributeNameArrayEditor(UI4.FormMaster.Editors.SortableArray.SortableArrayEditor):
    def addButtonCheckDragEvent(self, event): ...
    def addButtonDropEvent(self, event): ...

class AttributeNameEditor(StringFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def dragEnterEvent(self, event): ...
    def dropEvent(self, event): ...
    def setLocked(self, value, checkLockOps: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import PyFnAttribute
import typing
from PyUtilModule.WorkingSet import WorkingSet as WorkingSet
from _typeshed import Incomplete
from typing import Any, ClassVar, Set, Tuple

class LocationStateMap(dict):
    def __init__(self, parent: Incomplete | None = ...) -> None: ...
    def _LocationStateMap__tokenizeLocationPath(self, locationPath: str) -> list[str]: ...
    def deleteSubTree(self, locationPath: str, deleteRoot: bool = ...): ...
    def get(self, key: str, default: Incomplete | None = ...) -> Any: ...
    def getNearestSubTree(self, locationPath: str) -> Tuple[LocationStateMap, str]: ...
    def getSubTree(self, locationPath: str) -> LocationStateMap | None: ...
    def items(self, rootPath: Incomplete | None = ...) -> typing.Iterator: ...
    def keys(self, rootPath: Incomplete | None = ...) -> typing.Iterator: ...
    def pop(self, key: str, *args) -> Any: ...
    def values(self, rootPath: Incomplete | None = ...) -> typing.Iterator: ...
    def __contains__(self, key: str) -> bool: ...
    def __delitem__(self, key: str): ...
    def __getitem__(self, key: str) -> Any: ...
    def __iter__(self) -> typing.Iterator: ...
    def __len__(self, rootPath: Incomplete | None = ...) -> int: ...
    def __setitem__(self, key: str, value: object) -> None: ...

class WorkingSet:
    class State:
        Empty: ClassVar[int] = ...
        Excluded: ClassVar[int] = ...
        ExcludedWithChildren: ClassVar[int] = ...
        Included: ClassVar[int] = ...
        IncludedWithChildren: ClassVar[int] = ...
        @classmethod
        def coerceToBitmask(cls, bitmaskOrStates: int | list[int | None]) -> int: ...
        @classmethod
        def fromStateString(cls, stateString: str) -> int | None: ...
        @classmethod
        def getBitmaskFromStates(cls, states: list[int]) -> int: ...
        @classmethod
        def getStateString(cls, state: int) -> str: ...
        @classmethod
        def getStatesFromBitmask(cls, bitmask: int) -> list[int]: ...
        @classmethod
        def getValidStates(cls) -> list[int]: ...
        @classmethod
        def getValidStatesBitmask(cls) -> int: ...
        @classmethod
        def validate(cls, state: int): ...
    def __init__(self) -> None: ...
    def _WorkingSet__checkCallback(self, callback: typing.Callable, expectedArgNames: tuple[str, ...]): ...
    def _callAllowedStatesChangedCallbacks(self, sender): ...
    def _callLocationStateChangedCallbacks(self, locationStateChanges, sender): ...
    def _callWorkingSetClearedCallbacks(self): ...
    def addAllowedStatesChangedCallback(self, callback: typing.Callable): ...
    def addLocationStateChangedCallback(self, callback: typing.Callable): ...
    def addWorkingSetClearedCallback(self, callback: typing.Callable): ...
    def asGroupAttribute(self) -> PyFnAttribute.GroupAttribute: ...
    def clear(self, clearAllowedStates: bool = ..., sender: Incomplete | None = ..., useCallbacks: bool = ...) -> bool: ...
    def clearLocations(self, locationPathOrPaths: str | list[str], clearAllowedStates: bool = ..., clearGiven: bool = ..., clearChildren: bool = ..., sender: Incomplete | None = ..., useCallbacks: bool = ...) -> bool: ...
    def containsLocation(self, locationPath: str) -> bool: ...
    def excludeLocations(self, locationPathOrPaths: str | list[str], allowedStates: Incomplete | None = ..., sender: Incomplete | None = ...) -> bool: ...
    def excludeLocationsWithChildren(self, locationPathOrPaths: str | list[str], allowedStates: Incomplete | None = ..., sender: Incomplete | None = ...) -> bool: ...
    @classmethod
    def fromGroupAttribute(cls, groupAttribute: PyFnAttribute.GroupAttribute, sender: Incomplete | None = ..., useCallbacks: bool = ..., workingSetInstance: Incomplete | None = ...) -> WorkingSet: ...
    def getAllowedStates(self) -> list[int]: ...
    def getLocationAllowedStates(self, locationPath: str) -> list[int]: ...
    def getLocationState(self, locationPath: str) -> int: ...
    def getLocations(self, rootPath: Incomplete | None = ...) -> list[str]: ...
    def getLocationsAndStates(self, rootPath: Incomplete | None = ...) -> list[tuple[str, int]]: ...
    def getLocationsByState(self, state: int, rootPath: Incomplete | None = ...) -> list[str]: ...
    def getMinimumAllowedStateForLocation(self, locationPath: str) -> int | None: ...
    def getNumLocations(self, rootPath: Incomplete | None = ...) -> int: ...
    def getRootLocations(self) -> list[str]: ...
    def includeLocations(self, locationPathOrPaths: str | list[str], allowedStates: Incomplete | None = ..., sender: Incomplete | None = ...) -> bool: ...
    def includeLocationsWithChildren(self, locationPathOrPaths: str | list[str], allowedStates: Incomplete | None = ..., sender: Incomplete | None = ...) -> bool: ...
    def isIncludedLeafLocation(self, locationPath: str) -> bool: ...
    def isStateAllowed(self, state: int) -> bool: ...
    def isStateAllowedForLocation(self, locationPath: str, state: int) -> bool: ...
    def iterateLocationAncestors(self, locationPath: str, includeLocation: bool = ...) -> typing.Iterator: ...
    def matchesAnyChildren(self, locationPath: str, checkInheritance: bool = ...) -> bool: ...
    def matchesChildrenByInheritance(self, locationPath: str) -> bool: ...
    def matchesLocation(self, locationPath: str) -> bool: ...
    def removeAllCallbacks(self): ...
    def removeAllowedStatesChangedCallback(self, callback: typing.Callable): ...
    def removeLocationStateChangedCallback(self, callback: typing.Callable): ...
    def removeWorkingSetClearedCallback(self, callback: typing.Callable): ...
    def setAllowedStates(self, allowedStates: int | list[int | None], sender: Incomplete | None = ...) -> bool: ...
    def setLocationAllowedStates(self, locationPathOrPaths: str | list[str], allowedStates: int | list[int | None], sender: Incomplete | None = ...) -> bool: ...
    def setLocationStates(self, locationPathOrPaths: str | list[str], state: int, allowedStates: Incomplete | None = ..., sender: Incomplete | None = ..., useCallbacks: bool = ...) -> bool: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import KatanaResources as KatanaResources
import NodegraphAPI as NodegraphAPI
import UI4.FormMaster.ParameterKeyMimeData as ParameterKeyMimeData
import PyQt5.QtCore
import QT4Widgets as QT4Widgets
import QT4Widgets.Timebar
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import UI4.FormMaster.ParameterCurveEditSet
import Utils as Utils
from UI4.FormMaster.ParameterCurveEditSet import ParameterCurveEditWatcher as ParameterCurveEditWatcher
from UI4.Util.UndoGrouping import UndoContextGuard as UndoContextGuard
from typing import ClassVar, Set, Tuple

class TimelineWidget(QT4Widgets.Timebar.TimebarWidget, UI4.FormMaster.ParameterCurveEditSet.ParameterCurveEditWatcher):
    keysChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent) -> None: ...
    def _TimelineWidget__contextMenu(self, localPos): ...
    def _TimelineWidget__currentTime_changed(self, time, final): ...
    def _TimelineWidget__fullRange_changed(self, inTime, outTime): ...
    def _TimelineWidget__getNameForParameters(self, parameters): ...
    def _TimelineWidget__increment_changed(self, increment): ...
    def _TimelineWidget__nodegraph_loadEnd_callback(self, eventType, eventID, **kwargs): ...
    def _TimelineWidget__nodegraph_setCurrentTime_callback(self, eventType, eventID, **kwargs): ...
    def _TimelineWidget__nodegraph_setTimeIncrement_callback(self, eventType, eventID, **kwargs): ...
    def _TimelineWidget__nodegraph_setTimeRange_callback(self, eventType, eventID, **kwargs): ...
    def _TimelineWidget__nodegraph_setWorkingTimeRange_callback(self, eventType, eventID, **kwargs): ...
    def _TimelineWidget__on_keysChanged(self): ...
    def _TimelineWidget__selection_changed(self): ...
    def _TimelineWidget__showDopeSheet(self): ...
    def _TimelineWidget__timeRange_changed(self, inTime, outTime): ...
    def _TimelineWidget__updateKeys(self): ...
    def _initializeSet(self, curveEditSetCopy): ...
    def _keysActiveChanged(self, paramFullName, active): ...
    def _keysChanged(self, paramFullName): ...
    def _nameChanged(self, oldParamFullName, newParamFullName, newParamLocalName): ...
    def _nodeNameChanged(self, oldNodeName, newNodeName): ...
    def _showKeys(self, paramFullName, show): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnGeolibServices.ArgsFile as ArgsFile
import RenderingAPI as RenderingAPI
import Utils as Utils
import PyXmlIO as XmlIO
import typing
from typing import Set, Tuple

class Error(Exception): ...

def GetShaderArgsFileDirs(shaderName: str, rendererInfoPluginName: str) -> list[str]: ...
def GetShaderArgsFilePath(shaderName: str, rendererInfoPluginName: str) -> str | None: ...
def WriteArgsFileFromAttrAndHints(filePathOrObject: str | object, parameterNames: typing.Sequence[str], hintDict: dict[str, dict], containerHintDict: dict[str, dict], includePageInfo: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import RenderingAPI as RenderingAPI
import Utils as Utils
import Utils.UndoStack
import _CatalogItemProjectSettings
from CatalogItem import CatalogItem as CatalogItem
from typing import ClassVar, Set, Tuple

kPrimaryView: int
kSecondaryView: int

class _CatalogItemProjectSettings:
    class _GSV(tuple):
        _field_defaults: ClassVar[dict] = ...
        _fields: ClassVar[tuple] = ...
        _fields_defaults: ClassVar[dict] = ...
        def __init__(self, _cls, name, value, enabled) -> None: ...
        def _asdict(self): ...
        @classmethod
        def _make(cls, iterable): ...
        def _replace(self, _self, **kwds): ...
        def __getnewargs__(self): ...
        @property
        def enabled(self): ...
        @property
        def name(self): ...
        @property
        def value(self): ...
    def __init__(self, catalogItem: CatalogItem) -> None: ...
    @staticmethod
    def gsvsToString(gsvs: list[_CatalogItemProjectSettings._GSV]) -> str: ...
    @property
    def gsvs(self): ...
    @property
    def startFrame(self): ...

class _ClientSlotUpdateUndoEntry(Utils.UndoStack.UndoEntry):
    def __init__(self, clientKey: int, view: int, prevCatalogItem: CatalogItem, newCatalogItem: CatalogItem) -> None: ...
    def _ClientSlotUpdateUndoEntry__setClientCatalogItem(self, catalogItem: CatalogItem): ...
    def redo(self): ...
    def undo(self): ...

class _RootNodeProjectSettings:
    def __init__(self, rootNode: NodegraphAPI.GroupNode) -> None: ...
    @staticmethod
    def _RootNodeProjectSettings__canMatchGSV(param: NodegraphAPI.Parameter, gsv: _CatalogItemProjectSettings._GSV) -> bool: ...
    def applyCatalogItemFrameTime(self, catalogItemProjectSettings: _CatalogItemProjectSettings): ...
    def applyCatalogItemGSVs(self, catalogItemProjectSettings: _CatalogItemProjectSettings) -> list[_CatalogItemProjectSettings._GSV]: ...

def ApplyCatalogItemProjectSettings(catalogItem: CatalogItem): ...
def AutoViewCatalogItem(catalogItem): ...
def CreateCatalogClientKey(): ...
def DeleteCatalogClientKey(clientKey): ...
def GetAllowedViewSlots(): ...
def GetClientCatalogItems(clientKey): ...
def GetClientKeys(): ...
def GetClientViewSlots(clientKey): ...
def GetCurrentlyViewedSlots(): ...
def GetPrimaryClientKey(): ...
def GetRecommendedRenderSlots(nodeNames): ...
def GetUnassignedSlot(): ...
def IsClientPinned(clientKey: int) -> bool: ...
def SetClientCatalogItem(clientKey: int, view: int, catalogItem: CatalogItem.CatalogItem | None, isUndoable: bool = ...): ...
def SetClientPinned(clientKey: int, isPinned: bool): ...
def SetClientViewSlot(clientKey, view, slot): ...
def SwapClientViewSlots(clientKey): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import ApplyWhenOptions as ApplyWhenOptions, ApplyWhereOptions as ApplyWhereOptions, ExecutionModeOptions as ExecutionModeOptions
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import UI4.FormMaster.ParameterPolicy as ParameterPolicy
import UI4.FormMaster.ParameterPolicy
import Utils as Utils
from UI4.FormMaster.GenericAssignParameterPolicy import GenericAssignParameterPolicy as GenericAssignParameterPolicy
from UI4.FormMaster.GroupParameterPolicy import GroupParameterPolicy as GroupParameterPolicy
from UI4.FormMaster.ScalarParameterPolicy import ScalarParameterPolicy as ScalarParameterPolicy
from typing import Set, Tuple

class GraphStateVariableValuePolicy(GenericAssignParameterPolicy):
    def __init__(self, param, parent) -> None: ...
    def _GraphStateVariableValuePolicy__optionsChanged(self, event): ...
    def _appendToWrenchMenu(self, menu): ...
    def _freeze(self): ...
    def _thaw(self): ...
    def getWidgetHints(self): ...
    def setValue(self, value, final: bool = ...): ...
    def shouldDisplayWrench(self) -> bool: ...

class RootNodeParameterPolicyDelegate(UI4.FormMaster.ParameterPolicy.ParameterPolicyDelegate):
    def createPolicy(self, parentPolicy, param): ...

class VariableSwitchParameterPolicyDelegate(UI4.FormMaster.ParameterPolicy.ParameterPolicyDelegate):
    def createPolicy(self, parentPolicy, param): ...

class VariableSwitchPatternValuePolicy(ScalarParameterPolicy):
    def _appendToWrenchMenu(self, menu): ...
    def shouldDisplayWrench(self): ...

class VariableSwitchPatternsGroupValuePolicy(GroupParameterPolicy):
    def _appendToWrenchMenu(self, menu): ...
    def shouldDisplayWrench(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets as QT4FormWidgets
import Utils as Utils
from QT4FormWidgets.GroupFormWidget import GroupFormWidget
from typing import Set, Tuple

class GraphStateVariablesFormWidget(GroupFormWidget):
    def __init__(self, parent, valuePolicy, widgetFactory) -> None: ...
    def _GraphStateVariablesFormWidget__addVariable(self): ...
    def _GraphStateVariablesFormWidget__deleteAllVariables(self): ...
    def _GraphStateVariablesFormWidget__on_menuButton_menu(self, menu): ...
    def _lockChanged(self, state): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from typing import Set, Tuple, overload

class Element:
    def __init__(self, arg0: str) -> None: ...
    @overload
    def addChild(self, arg0: Element) -> Element: ...
    @overload
    def addChild(self, arg0: Element, arg1: int) -> Element: ...
    def addText(self, arg0: str) -> None: ...
    def copy(self) -> Element: ...
    def deleteAttr(self, arg0: str) -> None: ...
    def getAttr(self, arg0: str) -> str: ...
    def getAttrDict(self) -> dict: ...
    def getAttrNames(self) -> list[str]: ...
    def getChild(self, arg0: int) -> Element: ...
    def getElementType(self) -> str: ...
    def getNumberOfChildren(self) -> int: ...
    def getOpaqueElement(self) -> OpaqueElement: ...
    def getText(self, arg0: int) -> str: ...
    def hasAttr(self, arg0: str) -> bool: ...
    def hasText(self, arg0: int) -> bool: ...
    def isTypeOf(self, arg0: str) -> bool: ...
    def removeAllChildren(self) -> None: ...
    def removeChild(self, arg0: int) -> None: ...
    def setAttr(self, arg0: str, arg1: object) -> None: ...
    def setElementType(self, arg0: str) -> None: ...
    def setText(self, arg0: str, arg1: int) -> None: ...
    @overload
    def write(self, file: str, outputStyles=...) -> None: ...
    @overload
    def write(self, file: object, outputStyles=...) -> None: ...
    def writeString(self, outputStyles=...) -> str: ...
    def __bool__(self) -> bool: ...
    def __contains__(self, other: str) -> bool: ...
    def __delitem__(self, other: str) -> None: ...
    def __getitem__(self, index: str) -> str: ...
    def __iter__(self) -> Element_iter: ...
    def __len__(self) -> int: ...
    def __setitem__(self, index: str, object: object) -> None: ...

class Element_iter:
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self): ...

class OpaqueElement:
    def __init__(self) -> None: ...
    def getElement(self, *args, **kwargs): ...
    def getSize(self) -> int: ...
    def getUncompressedSize(self) -> int: ...

class OutputStyles:
    def __init__(self) -> None: ...
    def addToAttrOrder(self, arg0: str, arg1: str) -> None: ...
    def clearAttrOrder(self, arg0: str) -> None: ...
    def deleteStyle(self, arg0: str) -> None: ...
    def getAttrOrder(self, arg0: str) -> list[str]: ...
    def getStyleNames(self) -> list[str]: ...
    def hasStyle(self, arg0: str) -> bool: ...

class Parser:
    def __init__(self) -> None: ...
    def getParseDepthLimit(self) -> int: ...
    def getParsedByteOffsetBegin(self, arg0: Element) -> int: ...
    def getParsedByteOffsetEnd(self, arg0: Element) -> int: ...
    def isByteOffsetRecordingEnabled(self) -> bool: ...
    def isTextParsingEnabled(self) -> bool: ...
    @overload
    def parse(self, arg0: str) -> Element: ...
    @overload
    def parse(self, arg0: object) -> Element: ...
    def parseString(self, arg0: str) -> Element: ...
    def setByteOffsetRecordingEnabled(self, arg0: bool) -> None: ...
    def setParseDepthLimit(self, arg0: int) -> None: ...
    def setTextParsingEnabled(self, arg0: bool) -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class TreeEditor(PyQt5.QtWidgets.QLineEdit):
    edited: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, item, column, rect, value) -> None: ...
    def _TreeEditor__finished(self, accepted): ...
    def focusOutEvent(self, event): ...
    def keyPressEvent(self, event): ...

class TreeItem:
    def __init__(self, parent, name: str = ..., index: int = ...) -> None: ...
    def _TreeItem__setParent(self, parent, index): ...
    def _TreeItem__unselectChildren(self): ...
    def _TreeItem__updateDescendentInfo(self): ...
    def _setTreeWidget(self, treeWidget): ...
    def addChild(self, child, index: int = ...): ...
    def childAtY(self, y): ...
    def childByIndex(self, index): ...
    def childByName(self, name): ...
    def childCount(self): ...
    def childY(self, item): ...
    def children(self): ...
    def depth(self): ...
    def desiredColumnWidth(self, column): ...
    def expandable(self): ...
    def expanded(self): ...
    def getSortIndex(self): ...
    def hasChildren(self): ...
    def height(self): ...
    def index(self): ...
    def itemCollapsed(self): ...
    def itemDoubleClicked(self, column): ...
    def itemExpanded(self): ...
    def itemUpdated(self, updateBelow: bool = ..., emitSignals: bool = ...): ...
    def iterChildren(self): ...
    def name(self): ...
    def paint(self, painter, rect, palette, isCurrent, columnWidths): ...
    def paintCell(self, painter, rect, palette, column): ...
    def parent(self): ...
    def pixmap(self, column): ...
    def selected(self): ...
    def setExpandable(self, flag): ...
    def setExpanded(self, flag): ...
    def setHeight(self, height): ...
    def setName(self, name): ...
    def setPixmap(self, column, pixmap): ...
    def setSelected(self, flag, emitSignals: bool = ...): ...
    def setText(self, column, text): ...
    def setTextElideMode(self, column, mode): ...
    def sortChildren(self): ...
    def takeChild(self, index): ...
    def takeChildren(self): ...
    def text(self, column): ...
    def textElideMode(self, column): ...
    def treeWidget(self): ...
    def updateVisibleDescendentHeight(self, walk: bool = ...): ...
    def visibleChildren(self): ...
    def visibleDescendentHeight(self): ...

class TreeWidget(PyQt5.QtWidgets.QAbstractScrollArea):
    itemCollapsedSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    itemExpandedSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    itemSelectionChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    kDisclosureTriangleSize: ClassVar[int] = ...
    valid_event_keys: ClassVar[frozenset] = ...
    def __init__(self, *args) -> None: ...
    def _TreeWidget__addItemAndChildren(self, item, itemSet, checkFn): ...
    def _TreeWidget__columnResize(self, *args): ...
    def _TreeWidget__createPixmaps(self): ...
    def _TreeWidget__drawItem(self, item, rect, palette, p: Incomplete | None = ...): ...
    def _TreeWidget__drawItemChildren(self, p, rect, item, palette, x, y): ...
    def _TreeWidget__ensureVisible(self, rect): ...
    def _TreeWidget__findNextItem(self, item): ...
    def _TreeWidget__findPrevItem(self, item): ...
    def _TreeWidget__getPainter(self): ...
    def _TreeWidget__idle_callback(self, *args, **kwargs): ...
    def _TreeWidget__intersectingItems(self, rect, item, x, y, intersectList): ...
    def _TreeWidget__itemAtY(self, y): ...
    def _TreeWidget__itemRect(self, item): ...
    def _TreeWidget__positionHeader(self): ...
    def _TreeWidget__verticalActionTriggered(self, action): ...
    def _buildRootItem(self, parent, name): ...
    def clear(self): ...
    def columnAtPosition(self, x): ...
    def columnStartPosition(self, col): ...
    def columnWidths(self): ...
    def contentsPos(self): ...
    def contentsSize(self): ...
    def contentsToViewport(self, pos): ...
    def drawItems(self, p, rect): ...
    def event(self, event): ...
    def getCellRect(self, item, column): ...
    def getFullWidth(self): ...
    def getMultiSelect(self): ...
    def header(self): ...
    def indentation(self): ...
    def intersectingItems(self, rect): ...
    def itemAtPosition(self, pos): ...
    def itemCollapsed(self, item): ...
    def itemExpanded(self, item): ...
    def itemUpdated(self, item, updateBelow: bool = ..., emitSignals: bool = ...): ...
    def keyPressEvent(self, event): ...
    def keyReleaseEvent(self, event): ...
    def mouseDoubleClickEvent(self, event): ...
    def mouseMoveEvent(self, event): ...
    def mousePressEvent(self, event): ...
    def mouseReleaseEvent(self, event): ...
    def paintEvent(self, event): ...
    def popupEdited(self, item, column, value): ...
    def popupEditor(self, item, column, value: Incomplete | None = ...): ...
    def resizeEvent(self, event): ...
    def rootItem(self): ...
    def scrollContentsBy(self, dx, dy): ...
    def scrollToItem(self, item): ...
    def selectAll(self): ...
    def selection(self): ...
    def setContentsPos(self, pos): ...
    def setCurrentItem(self, item): ...
    def setHeaderLabels(self, labels): ...
    def setHeaderVisible(self, state): ...
    def setIndentation(self, indentation): ...
    def setMultiSelect(self, state): ...
    def setSelection(self, newSelection): ...
    def updateContentsSize(self, item: Incomplete | None = ...): ...
    def updateContentsSizeOnIdle(self): ...
    def updateScrollBars(self, scrollContents: bool = ...): ...
    def updateVisibleDescendentHeightOnIdle(self, item): ...
    def viewportToContents(self, pos): ...
    def wheelEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class CapsuleCombo(CapsuleComboBase, PyQt5.QtWidgets.QWidget):
    valueChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, **kwargs) -> None: ...
    def _getFontMetrics(self): ...
    def _getWidgetRect(self): ...
    def _itemsChanged(self): ...
    def _numItemsChanged(self): ...
    def enterEvent(self, ev): ...
    def leaveEvent(self, ev): ...
    def mouseMoveEvent(self, ev): ...
    def mousePressEvent(self, ev): ...
    def paintEvent(self, ev): ...
    def sizePolicy(self): ...

class CapsuleComboBase:
    def __init__(self, exclusive: bool = ..., equalPartitionWidths: bool = ..., minPartitionWidth: int = ..., horizontalMargin: int = ..., verticalMargin: int = ..., interMargin: int = ..., cornerRoundness: int = ..., delimiter: str = ...) -> None: ...
    def _doMousePress(self, x, y, ev): ...
    def _doPaint(self, p, palette): ...
    def _getFontMetrics(self): ...
    def _getWidgetRect(self): ...
    def _itemsChanged(self): ...
    def _numItemsChanged(self): ...
    def addItem(self, text, bgColor, enabled, meta: Incomplete | None = ...): ...
    def clearItems(self): ...
    def getAlignment(self): ...
    def getAreaBoundaries(self): ...
    def getBubbleIndexAtPosition(self, px, py): ...
    def getEnabledItemIndices(self): ...
    def getEnabledItems(self): ...
    def getItemIndex(self, meta): ...
    def getItems(self): ...
    def getNumItems(self): ...
    def getRawTextWidths(self): ...
    def getReadOnly(self): ...
    def isExclusive(self): ...
    def isItemEnabled(self, index): ...
    def joinEnabledItems(self, items): ...
    def minimumSizeHint(self): ...
    def setAlignment(self, alignment): ...
    def setEnabledItems(self, metaList): ...
    def setExclusive(self, exclusive): ...
    def setIndexUnderCursor(self, index): ...
    def setItemEnabled(self, index, isEnabled): ...
    def setOptions(self, options, colors: Incomplete | None = ..., displayText: Incomplete | None = ...): ...
    def setReadOnly(self, readOnly): ...
    def setTransitionValidator(self, validator): ...
    def sizeHint(self): ...
    def splitNonExclusiveValueString(self, value): ...

class TransitionValidator:
    def fixup(self, newValue, oldValue): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFCurve as PyFCurve
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QTFCurve.MixedGroupState import MixedGroupState as MixedGroupState
from QTFCurve.UniqueListDict import UniqueListDict as UniqueListDict
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class FCurveListView(PyQt5.QtWidgets.QTreeWidget):
    class _FCurveContainerObserver(PyFCurve.ContainerObserver):
        def __init__(self, listView, selection) -> None: ...
        def addCurve(self, curve): ...
        def containerChanged(self, container): ...
        def removeCurve(self, curve): ...

    class _FCurveItem(PyQt5.QtWidgets.QTreeWidgetItem):
        class _FCurveGroupItem(PyQt5.QtWidgets.QTreeWidgetItem):
            def __init__(self, parent, name) -> None: ...
            def clicked(self, col, fromMixed: bool = ...): ...
            def getCurve(self): ...
            def isGroup(self): ...
            def propagateState(self, toVis, toLocked, fromVis, fromLocked): ...
        def __init__(self, parent, curve) -> None: ...
        def _FCurveItem__createGroupItem(self, parent, curveGroups): ...
        def clicked(self, col, fromMixed: bool = ...): ...
        def getCurve(self): ...
        def isGroup(self): ...
        def propagateRemoval(self): ...
        def refreshAllProperties(self, oldHidden, oldLocked): ...

    class _FCurveListViewObserver(PyFCurve.FCurveObserver):
        def __init__(self, listView, selection) -> None: ...
        def curvePropertyChanged(self, fc, key, value): ...
    unloadCurve: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, container, selection, parent: Incomplete | None = ...) -> None: ...
    def _FCurveListView__colClicked(self, item, col): ...
    def _FCurveListView__updateSelection(self): ...
    def addCustomColumn(self, name, default, valuelist: list = ...): ...
    def child(self, index): ...
    def childCount(self): ...
    def createCurve(self): ...
    def customColumnNames(self) -> list[str]: ...
    def keyPressEvent(self, e): ...
    def mouseDoubleClickEvent(self, e): ...
    def mousePressEvent(self, e): ...
    def removeCurves(self, removeSelection: bool = ...): ...
    def renameCurve(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.ArrayItemEntry as ArrayItemEntry
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
from QT4FormWidgets.FWidget import FBoxLayout as FBoxLayout, FButton as FButton, FDisclosureTriangle as FDisclosureTriangle, FLabel as FLabel, FLockIcon as FLockIcon, FMenu as FMenu, FPixmap as FPixmap, FSpacer as FSpacer, FStateBadge as FStateBadge, FSvgIcon as FSvgIcon, FToggleStateBadge as FToggleStateBadge, FWidget as FWidget
from QT4FormWidgets.FormWidget import AlignChildLabelWidths as AlignChildLabelWidths, AlignLeftControlWidths as AlignLeftControlWidths, FormWidget as FormWidget, ScrubbingStates as ScrubbingStates
from QT4FormWidgets.GroupFormWidget import GroupFormWidget as GroupFormWidget
from QT4FormWidgets.InputWidgets import InputLineEdit as InputLineEdit
from typing import Set, Tuple

class MultiFormWidget(GroupFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _MultiFormWidget__doValueEdit(self): ...
    def _MultiFormWidget__freezeEntries(self): ...
    def _MultiFormWidget__syncArrayValues(self): ...
    def _buildControlWidget(self, controlLayout): ...
    def _buildLabel(self, labelText, pos: int = ...): ...
    def _freeze(self): ...
    def _lockChanged(self, state): ...
    def _participatesInLabelAlignment(self): ...
    def _thaw(self): ...
    def showPopdown(self, value): ...
    def valueChangedEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import AttributeHints as AttributeHints, AttributePolicy as AttributePolicy, DocDump as DocDump, EditorCache as EditorCache, Editors as Editors, EnableableParameterPolicy as EnableableParameterPolicy, FnAttributePolicy as FnAttributePolicy, GenericAssignParameterPolicy as GenericAssignParameterPolicy, GenericAssignParameterPolicy_geolib3 as GenericAssignParameterPolicy_geolib3, GraphStateVariablePolicy as GraphStateVariablePolicy, HintsDelegate as HintsDelegate, KatanaFactory as KatanaFactory, KatanaVisibilityOps as KatanaVisibilityOps, LiveAttributePolicy as LiveAttributePolicy, MaterialNodePolicy as MaterialNodePolicy, NodeActionDelegate as NodeActionDelegate, NodeHints as NodeHints, NodeMimeData as NodeMimeData, Pages as Pages, ParameterCurveEditSet as ParameterCurveEditSet, ParameterKeyMimeData as ParameterKeyMimeData, ParameterPolicy as ParameterPolicy, Scripts as Scripts, ShadingNodePolicy as ShadingNodePolicy, States as States, WrenchDelegate as WrenchDelegate
from UI4.FormMaster.ArrayParameterPolicy import ArrayParameterPolicy as ArrayParameterPolicy
from UI4.FormMaster.AttributePolicy import ArrayAttributePolicy as ArrayAttributePolicy, AttributeStateSource as AttributeStateSource, BaseAttributePolicy as BaseAttributePolicy, CreateAttributePolicy as CreateAttributePolicy, CreateProducerGroupPolicy as CreateProducerGroupPolicy, GroupAttributePolicy as GroupAttributePolicy, ScalarAttributePolicy as ScalarAttributePolicy
from UI4.FormMaster.BaseParameterPolicy import BaseParameterPolicy as BaseParameterPolicy
from UI4.FormMaster.EditorCache import NodeEditorCache as NodeEditorCache
from UI4.FormMaster.EnableableParameterPolicy import EnableableGroupParameterPolicy as EnableableGroupParameterPolicy, EnableableScalarParameterPolicy as EnableableScalarParameterPolicy, EnableableScalarPolicyDelegate as EnableableScalarPolicyDelegate
from UI4.FormMaster.GroupParameterPolicy import GroupParameterPolicy as GroupParameterPolicy
from UI4.FormMaster.HintsDelegate import FnMatchHintsDelegate as FnMatchHintsDelegate, RegisterAttributeHintsDelegate as RegisterAttributeHintsDelegate, RegisterHintsDelegate as RegisterHintsDelegate, SimpleDictHintsDelegate as SimpleDictHintsDelegate, UpdateAttributeHintsWithDelegates as UpdateAttributeHintsWithDelegates, UpdateHintsWithDelegates as UpdateHintsWithDelegates
from UI4.FormMaster.KatanaFactory import AttributeWidgetFactory as AttributeWidgetFactory, KatanaWidgetFactory as KatanaWidgetFactory, LoadAllEditors as LoadAllEditors, LookupWidgetType as LookupWidgetType, ParameterWidgetFactory as ParameterWidgetFactory, RegisterPluginWidget as RegisterPluginWidget
from UI4.FormMaster.LiveAttributePolicy import ArrayLiveAttributePolicy as ArrayLiveAttributePolicy, ClearCache as ClearCache, ConvertAttributePolicyToLive as ConvertAttributePolicyToLive, CreateProducerLiveGroupPolicy as CreateProducerLiveGroupPolicy, Flush as Flush, GroupLiveAttributePolicy as GroupLiveAttributePolicy, PageGroupLiveAttributePolicy as PageGroupLiveAttributePolicy, Pause as Pause, RegisterLiveAttributeChangedCallback as RegisterLiveAttributeChangedCallback, ScalarLiveAttributePolicy as ScalarLiveAttributePolicy, Unpause as Unpause
from UI4.FormMaster.NodeBypassPolicy import NodeBypassPolicy as NodeBypassPolicy
from UI4.FormMaster.ParameterCurveEditSet import ClearParameterCurveEditSet as ClearParameterCurveEditSet, GetParameterCurveEdited as GetParameterCurveEdited, ParameterCurveEditWatcher as ParameterCurveEditWatcher, SetParameterCurveEdited as SetParameterCurveEdited
from UI4.FormMaster.ParameterPolicy import CreateDefaultParameterPolicy as CreateDefaultParameterPolicy, CreateParameterPolicy as CreateParameterPolicy, IsParameterEnabled as IsParameterEnabled, ParameterPolicyDelegate as ParameterPolicyDelegate, RegisterPolicyDelegate as RegisterPolicyDelegate
from UI4.FormMaster.RenderNodeReferencePolicy import RenderNodeReferencePolicy as RenderNodeReferencePolicy
from UI4.FormMaster.ScalarParameterPolicy import ScalarParameterPolicy as ScalarParameterPolicy
from UI4.FormMaster.WrenchDelegate import RegisterWrenchDelegate as RegisterWrenchDelegate, UpdateWrenchMenuWithDelegates as UpdateWrenchMenuWithDelegates
from typing import Set, Tuple

ALLOWED_TYPES: tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import ResourceFiles as ResourceFiles
import UI4 as UI4
from QT4FormWidgets.FormWidget import FormWidget
from typing import Set, Tuple

class HtmlEditorFormWidget(FormWidget):
    def __init__(self, parent, valuePolicy, widgetFactory) -> None: ...
    def _HtmlEditorFormWidget__expressionChanged(self): ...
    def _HtmlEditorFormWidget__toggleExpressionState(self): ...
    def _HtmlEditorFormWidget__valueChanged(self, event): ...
    def _freeze(self): ...
    def _lockChanged(self, value): ...
    def _thaw(self): ...
    def cleanupError(self, error): ...
    def doReturnPressed(self): ...
    def generateMenu(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFCurveGraphWidget
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class BaseGradientLayerStackLayer(QT4GLLayerStack.LayerStack.Layer):
    class viewportObject:
        def __init__(self, bounds) -> None: ...
        def getWorldBounds(self): ...
    def paintGL(self): ...

class GradientLayerBundle:
    def __init__(self) -> None: ...
    def activate(self): ...
    def addForegroundLayer(self, layer): ...
    def deactivate(self): ...
    def getBackgroundLayer(self): ...
    def getForegroundLayers(self): ...
    def isActive(self): ...
    def setBackgroundLayer(self, layer): ...

class GradientLayerBundleManager:
    _BACKGROUND_COLORS: ClassVar[dict] = ...
    _CURVE_COLORS: ClassVar[dict] = ...
    _CURVE_FUNCTIONS: ClassVar[dict] = ...
    _GradientLayerBundleManager__gray_background_color: ClassVar[tuple] = ...
    _GradientLayerBundleManager__gray_curve_color: ClassVar[tuple] = ...
    def __init__(self) -> None: ...
    def _GradientLayerBundleManager__calcCursorValues(self, rgb): ...
    def _GradientLayerBundleManager__getCurrentCursorLayers(self): ...
    def _probeResult_CB(self, probeInfo): ...
    def _registerCurve(self, curveName, curveAndLayerInfo): ...
    def _setActiveCurves(self, curveNames): ...
    def _setCurveKey(self, value): ...
    def _updateGraphView(self): ...

class HueGradientDrawLayer:
    def __init__(self, showBox: bool = ..., showTopBar: bool = ..., showBottomBar: bool = ..., showVerticals: bool = ..., infiniteExtent: bool = ..., clampVertical: bool = ..., cursorRGBConvertFunction: Incomplete | None = ...) -> None: ...
    def _HueGradientDrawLayer__drawBar(self, colorKeys, y): ...
    def _HueGradientDrawLayer__drawBox(self, colorKeys, ymin, ymax, xmin, xmax): ...
    def _HueGradientDrawLayer__drawCursor(self, ymin, ymax): ...
    def _HueGradientDrawLayer__drawVerticals(self, colorKeys, ymin, ymax): ...
    def _HueGradientDrawLayer__rgbConvert(self, rgb): ...
    def _getColors(self, knots, colorInfo): ...
    def _getDomainKnots(self): ...
    def _setUpColors(self): ...
    def draw(self, viewport): ...
    def getCursorValue(self): ...
    def hide(self): ...
    def hideCursor(self): ...
    def isHidden(self): ...
    def setCursorActive(self, active): ...
    def setCursorRGBConvertFunction(self, fn): ...
    def setCursorValue(self, value): ...
    def setCursorValueFromRGB(self, rgb): ...
    def show(self): ...

class HueGradientFCurveLayer(HueGradientDrawLayer, PyFCurveGraphWidget.Layer):
    def __init__(self, *args, **kwargs) -> None: ...

class HueGradientLayerStackLayer(HueGradientDrawLayer, BaseGradientLayerStackLayer):
    def __init__(self, *args, **kwargs) -> None: ...

class LinearGradientDrawLayer(HueGradientDrawLayer):
    def __init__(self, darkColor, lightColor, **kwargs) -> None: ...
    def _LinearGradientDrawLayer__lerpAndAdjustColor(self, c0, c1, x, k): ...
    def _LinearGradientDrawLayer__smush(self, value, k): ...
    def _getColors(self, knots, colorInfo): ...
    def _getDomainKnots(self): ...
    def _setUpColors(self): ...
    def setLightColor(self, newLightColor): ...

class LinearGradientFCurveLayer(LinearGradientDrawLayer, PyFCurveGraphWidget.Layer):
    def __init__(self, *args, **kwargs) -> None: ...

class LinearGradientLayerStackLayer(LinearGradientDrawLayer, BaseGradientLayerStackLayer):
    def __init__(self, *args, **kwargs) -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.SceneGraphView.ItemDelegates.BaseItemDelegate
import UI4.Widgets.SceneGraphView.ItemDelegates.BaseItemDelegate
import UI4.Widgets.SceneGraphView.ItemDelegates.ParameterItemDelegate
import Utils as Utils
import typing
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class AttributeValuePopupWidget(PyQt5.QtWidgets.QWidget):
    BorderLineColor: ClassVar[PyQt5.QtGui.QColor] = ...
    MinimumPopupHeight: ClassVar[int] = ...
    MinimumPopupWidth: ClassVar[int] = ...
    Padding: ClassVar[int] = ...
    ShapePointPadding: ClassVar[int] = ...
    ShapePointWidth: ClassVar[int] = ...
    SizeHintHeightPadding: ClassVar[int] = ...
    SizeHintWidthPadding: ClassVar[int] = ...
    WindowFillColor: ClassVar[PyQt5.QtGui.QColor] = ...
    WindowFlags: ClassVar[PyQt5.QtCore.Qt.WindowType] = ...
    def __init__(self) -> None: ...
    def getActiveAreaRect(self) -> PyQt5.QtCore.QRect: ...
    def hideLater(self): ...
    def paintEvent(self, event: PyQt5.QtGui.QPaintEvent): ...
    def resizeToFitItemDelegate(self, sizeHint: PyQt5.QtCore.QSize): ...
    def show(self, index: PyQt5.QtCore.QModelIndex, itemDelegate: ParameterItemDelegate[UI4.Widgets.SceneGraphView.ItemDelegates.ParameterItemDelegate.ParameterItemDelegate], treeWidget: PyQt5.QtWidgets.QTreeWidget): ...

class AttributeValuePopupWidgetManager(PyQt5.QtCore.QObject):
    _instance: ClassVar[None] = ...
    def __init__(self) -> None: ...
    def eventFilter(self, obj, event): ...
    @classmethod
    def getInstance(cls) -> AttributeValuePopupWidgetManager: ...
    def hide(self): ...
    def setKeepVisible(self, keepVisible: bool): ...
    def show(self, index: PyQt5.QtCore.QModelIndex, itemDelegate: UI4.Widgets.SceneGraphView.ItemDelegates.ParameterItemDelegate, treeWidget: PyQt5.QtWidgets.QTreeWidget, keepVisible: bool = ...): ...

class ItemDelegateHostWidget(PyQt5.QtWidgets.QWidget):
    def __init__(self, index: Incomplete | None = ..., itemDelegate: Incomplete | None = ..., parent: Incomplete | None = ...) -> None: ...
    def _ItemDelegateHostWidget__on_model_dataChanged(self, startIndex: PyQt5.QtGui.QModelIndex, endIndex: PyQt5.QtGui.QModelIndex): ...
    def getItemDelegate(self) -> BaseItemDelegate[UI4.Widgets.SceneGraphView.ItemDelegates.BaseItemDelegate.BaseItemDelegate]: ...
    def paintEvent(self, event: PyQt5.QtGui.QPaintEvent): ...
    def setIndex(self, index: PyQt5.QtCore.QModelIndex): ...
    def setItemDelegate(self, itemDelegate: BaseItemDelegate[UI4.Widgets.SceneGraphView.ItemDelegates.BaseItemDelegate.BaseItemDelegate]): ...
    def setResizeCallback(self, callback: typing.Callable | None): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import ResourceFiles as ResourceFiles
from PyQt5.QtGui import EXPRESSION_TRIANGLE_COLOR as EXPRESSION_TRIANGLE_COLOR, TEXTCOLOR_ERROR as TEXTCOLOR_ERROR, TEXTCOLOR_WARNING as TEXTCOLOR_WARNING
from ResourceFiles.IconManager import ResourceManager as ResourceManager
from _typeshed import Incomplete
from typing import Set, Tuple

HIGHLIGHTSTYLE_ALWAYS: int
HIGHLIGHTSTYLE_NONE: int
HIGHLIGHTSTYLE_ROLLOVER: int
HIGHLIGHTSTYLE_TRANSPARENT: int
PALETTE_COLOR_ROLES_POLICY: list
PALETTE_COLOR_ROLES_READ_ONLY: list
STATE_BADGE_DEFAULT_SIZE: int
TOGGLE_STATE_BADGE_DEFAULT_SIZE: int

def ApplyPolicyPalette(policy: QT4FormWidgets.AbstractValuePolicy, widget: PyQt5.QtWidgets.QWidget, locked: Incomplete | None = ...): ...
def GetDefaultBoldLabelFont(): ...
def GetDefaultFixedPitchFont(): ...
def GetDefaultLabelFont(scale: Incomplete | None = ...): ...
def GetDefaultSecondaryLabelFont(): ...
def GetGroupLabelColor(palette: Incomplete | None = ...): ...
def GetHighlightColor(palette, on: bool = ..., active: bool = ...): ...
def GetPolicyPalette(policy: QT4FormWidgets.AbstractValuePolicy, locked: Incomplete | None = ...) -> PyQt5.QtGui.QPalette: ...
def GetReadOnlyPalette(locked: bool) -> PyQt5.QtGui.QPalette: ...
def GetStateBadgePixmap(letter, color, palette: Incomplete | None = ...): ...
def HighlightStyleValid(style): ...
def PaintBicolorRect(painter, rect, ulColor, lrColor): ...
def PaintCheckers(painter, rect, size: int = ..., lightValue: int = ..., darkValue: int = ...): ...
def PaintDisclosureTriangle(painter, _width, height, aspect, palette, rotation: Incomplete | None = ..., size: Incomplete | None = ..., outlineColor: Incomplete | None = ..., color: Incomplete | None = ..., hovered: bool = ...): ...
def PaintDragBar(painter, rect, multiplier: float = ...): ...
def PaintGroupEnclosureSidebar(painter, fullRect, color, end: bool = ..., sidebarWidth: int = ...): ...
def PaintHighlightBar(painter, fullRect, color, roundTop: bool = ..., roundBottom: bool = ...): ...
def PaintQLineEditText(painter, option, flags, text, leftAlignUnderflow: bool = ...): ...
def PaintStateBadge(painter, width, height, letter, color, palette: Incomplete | None = ..., glow: bool = ..., textColor: Incomplete | None = ..., size: int = ...): ...
def PaintToggleStateBadge(painter, width, height, color, palette: Incomplete | None = ..., glow: bool = ..., size: int = ...): ...
def UpdateFontCache(font): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.EnableableParameterPolicy as EnableableParameterPolicy
import UI4.FormMaster.HintsDelegate as HintsDelegate
import UI4.FormMaster.ParameterPolicy as ParameterPolicy
from UI4.FormMaster.NodeHints.Common2D import GetCapsuleRgbaDict as GetCapsuleRgbaDict
from typing import Set, Tuple

BloomAmountParamHints: dict
BloomEnableParamHints: dict
BloomGroupParamHints: dict
BloomModeParamHints: dict
BloomThresholdParamHints: dict
BorderExtendParamHints: dict
BorderOverscanParamHints: dict
_NodeHints: dict
_NodeName: str



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyQt5.QtGui
import QT4GLLayerStack as QT4GLLayerStack
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import UI4 as UI4
from QT4GLLayerStack.LayerStack import LayerStack
from UI4.Tabs.DopeSheet.KeyDrawingLayer import KeyDrawingLayer as KeyDrawingLayer
from UI4.Tabs.DopeSheet.KeyInteractionLayer import KeyInteractionLayer as KeyInteractionLayer
from UI4.Tabs.DopeSheet.PanAndScrollLayer import PanAndScrollLayer as PanAndScrollLayer
from UI4.Util.Events import ClickFilter as ClickFilter
from typing import ClassVar, Set, Tuple

class KeyLayerStack(LayerStack):
    _GL_UPDATE_THROTTLE_INTERVAL: ClassVar[int] = ...
    _MAX_SCALE_X: ClassVar[int] = ...
    def __init__(self, parent, treeWidget) -> None: ...
    def _KeyLayerStack__directUpdateGL(self): ...
    def _KeyLayerStack__getKeysSelected(self, item, keyList, includeParents: bool = ...): ...
    def _KeyLayerStack__getTreeOffsetAndHeight(self): ...
    def clearKeysPotential(self): ...
    def clearSelection(self, update: bool = ...): ...
    def contextMenuEvent(self, event): ...
    def event(self, rawEvent): ...
    def fontChange(self, oldFont: PyQt5.QtGui.QFont): ...
    def frameInOut(self): ...
    def frameKeysAll(self): ...
    def frameKeysSelected(self): ...
    def frameTimeRange(self, inTime, outTime): ...
    def frameWorkingInOut(self): ...
    def getKeysSelected(self, includeParents: bool = ...): ...
    def hitTestBox(self, worldBox): ...
    def hitTestPoint(self, worldPt): ...
    def hitTestPointForItem(self, worldPt): ...
    def resizeGL(self, w, h): ...
    def selectAll(self, update: bool = ...): ...
    def setGhostTime(self, t): ...
    def setKeysPotential(self, potentials, conflict: bool = ...): ...
    def setKeysSelected(self, keyList, selected, update: bool = ...): ...
    def setKeysSelectedExclusive(self, keyList, update: bool = ...): ...
    def toggleKeysSelected(self, keyList, update: bool = ...): ...
    def translateEyePoint(self, dx, dy): ...
    def updateGL(self): ...
    def updateScroll(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
from PyUtilModule.ScenegraphUtils import AbbreviateScenegraphLocations as AbbreviateScenegraphLocations
from UI4.Widgets.HorizontalDivider import HorizontalDivider as HorizontalDivider
from UI4.Widgets.MetaCheckBox import MetaCheckBox as MetaCheckBox
from UI4.Widgets.SceneGraphView.SceneGraphView import SceneGraphView
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class LightListPopup(PyQt5.QtWidgets.QFrame):
    _LightListPopup__BUTTON_PADDING: ClassVar[PyQt5.QtCore.QSize] = ...
    _LightListPopup__MARGIN: ClassVar[int] = ...
    _LightListPopup__MINIMUM_SIZE: ClassVar[PyQt5.QtCore.QSize] = ...
    _LightListPopup__TITLE: ClassVar[str] = ...
    aboutToShow: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    changed: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    hideSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    showSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, sceneGraphView, parent: Incomplete | None = ..., title: Incomplete | None = ...) -> None: ...
    def _LightListPopup__on_aboutToShow(self): ...
    def _LightListPopup__on_allLightsOff(self): ...
    def _LightListPopup__on_allLightsOn(self): ...
    def _LightListPopup__on_allLightsReset(self): ...
    def _LightListPopup__on_hide(self): ...
    def _LightListPopup__on_show(self): ...
    def _buildHeaderWidget(self): ...
    def _buildLocationsWidget(self): ...
    def _getHeaderWidget(self): ...
    def _getLocationsWidget(self): ...
    def getSceneGraphView(self) -> SceneGraphView: ...
    def hideEvent(self, event): ...
    def popup(self, globalPos): ...
    def showEvent(self, event): ...

class _LightListCheckBoxesWidget(PyQt5.QtWidgets.QScrollArea):
    _LightListCheckBoxesWidget__LIGHT_LIST_ATTRIBUTE: ClassVar[str] = ...
    _LightListCheckBoxesWidget__LIGHT_LIST_LOCATION: ClassVar[str] = ...
    _LightListCheckBoxesWidget__PADDING: ClassVar[PyQt5.QtCore.QSize] = ...
    changed: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, sceneGraphView, parent: Incomplete | None = ...) -> None: ...
    def _LightListCheckBoxesWidget__deleteCheckBoxes(self): ...
    def _LightListCheckBoxesWidget__emitChangedSignal(self): ...
    def _LightListCheckBoxesWidget__getLightLocationsFromAttributes(self, attributes): ...
    def _LightListCheckBoxesWidget__lightLocationsAttributeCallback(self, locationPath, attributes): ...
    def _LightListCheckBoxesWidget__on_checkBoxToggled(self, lightLocation, isOn): ...
    def _LightListCheckBoxesWidget__populateWidget(self): ...
    def _LightListCheckBoxesWidget__populateWidgetFromAttributes(self, attributes): ...
    def _LightListCheckBoxesWidget__updateWidgetLocations(self, locationNames, displayNames): ...
    def getSceneGraphView(self) -> SceneGraphView: ...
    def on_aboutToShow(self): ...
    def on_allLightsOff(self): ...
    def on_allLightsOn(self): ...
    def on_allLightsReset(self): ...
    def on_hide(self): ...
    def sizeHint(self): ...
    def suppressSignals(self, signalsSuppressed): ...

class _SignalSuppressor:
    def __init__(self, widget) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *args): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4Color.ColorPolicy as ColorPolicy
import QT4Color.Globals as Globals
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from _typeshed import Incomplete
from typing import Set, Tuple

def BuildSwatchPixmap(color, enableFilmlook: bool = ..., enableNoFilmlookColorSpace: bool = ..., hasAlpha: bool = ..., size: Incomplete | None = ...): ...
def BuildSwatchPixmapForPolicy(colorPolicy, size: Incomplete | None = ...): ...
def GetSwatchColors(color, enableFilmlook: bool = ..., enableNoFilmlookColorSpace: bool = ..., hasAlpha: bool = ...): ...
def PaintColorSwatch(painter, option, overBlack: Incomplete | None = ..., overWhite: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyFCurve as PyFCurve
import PyFCurveGraphWidget
import QT4Widgets as QT4Widgets
import QTFCurve as QTFCurve
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from PyUtilModule.VirtualKatana import FormMaster as FormMaster
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from _typeshed import Incomplete
from typing import Set, Tuple

PluginRegistry: list
TEMPORAL_QUANTIZATION_STEPS: int
TEMPORAL_QUANTIZATION_STEPS_INVERSE: float
global_paramExpressionCache: dict
global_paramExpressionCurves: dict
global_paramsShowingKeys: set

class CurveEditorPanel(BaseTab):
    class _LocalDomainSelectionObserver(PyFCurveGraphWidget.DomainSliderObserver):
        def __init__(self, domainSlider) -> None: ...
        def valueChanged(self, domainSlider: Incomplete | None = ...): ...
    def __init__(self, parent) -> None: ...
    def _CurveEditorPanel__actionBegin_callback(self, name): ...
    def _CurveEditorPanel__actionEnd_callback(self): ...
    def _CurveEditorPanel__finalizeValue_self_callback(self, eventType, eventID, **kwargs): ...
    def _CurveEditorPanel__nodegraph_setCurrentTime_callback(self, eventType, eventID, **kwargs): ...
    def _CurveEditorPanel__parameter_showKeys_callback(self, eventType, eventID, paramFullName, show): ...
    def _CurveEditorPanel__processUnfocusedKeyEvent(self, unfocusedEvent): ...
    def _CurveEditorPanel__unloadCurve(self, fcurve): ...
    def _CurveEditorPanel__updateDomainSlider(self): ...
    def _CurveEditorPanel__updateKeys(self): ...
    def _CurveEditorPanel__update_self_callback(self, eventType, eventID, **kwargs): ...
    def closeEvent(self, event): ...
    def event(self, event): ...

def __eval_node_param(name, frame): ...
def global_parameter_showKeys_callback(eventType, eventID, paramFullName, show): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyXmlIO as PyXmlIO
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.SortablePanel as SortablePanel
import UI4 as UI4
import UI4.FormMaster.Editors.UserParametersDialogs
import UI4.Widgets.SortablePanel
import Naming as UniqueName
import Utils as Utils
from UI4.FormMaster.Editors.UserParametersDialogs import ParameterRenameDialog as ParameterRenameDialog
from typing import Set, Tuple

class CopyChildrenDialog(PyQt5.QtWidgets.QDialog):
    def __init__(self, children) -> None: ...
    def _CopyChildrenDialog__selectionChanged(self): ...
    def accept(self): ...

class DynamicTemplateGroupFormWidget(UI4.Widgets.SortablePanel.SortablePanelFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _DynamicTemplateGroupFormWidget__addTemplate(self, templateName): ...
    def _DynamicTemplateGroupFormWidget__copy(self): ...
    def _DynamicTemplateGroupFormWidget__paste(self): ...
    def _childPanelsShouldAddWrench(self): ...
    def alignChildLabelWidths(self): ...
    def buildAddMenu(self, menu): ...
    def panelDeleted(self, index): ...
    def panelReordered(self, oldPos, newPos): ...
    def showPopdown(self, value): ...
    def updatePanels(self): ...

class TemplateGroupPanel(UI4.Widgets.SortablePanel.SortablePanelBase):
    def __init__(self, parent, name, policy, widgetFactory, addWrench) -> None: ...
    def _TemplateGroupPanel__getLabelBaseName(self, inputName): ...
    def _TemplateGroupPanel__nameChangedEvent(self, event): ...
    def _TemplateGroupPanel__policyNameChanged(self, event): ...
    def _TemplateGroupPanel__renameAction(self): ...
    def _TemplateGroupPanel__setName(self, value): ...
    def _TemplateGroupPanel__wrenchMenuAboutToShow(self): ...
    def _freeze(self): ...
    def _thaw(self): ...
    def getValuePolicy(self): ...
    def getWidget(self): ...
    def hideEvent(self, event): ...
    def showEvent(self, event): ...

class TemplateInstanceNameDialog(UI4.FormMaster.Editors.UserParametersDialogs.ParameterRenameDialog):
    def __init__(self, startingName, peerNames, templateType) -> None: ...
    def _getAdjustedFieldValue(self, text): ...
    def _getAdjustedUniqueDisplayName(self, text): ...
    def _isTextValid(self, text): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.Editors.PolicyFindPopup as PolicyFindPopup
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
from QT4FormWidgets.GroupFormWidget import GroupFormWidget
from typing import Set, Tuple

class FindingGroupFormWidget(GroupFormWidget):
    def _buildLock(self, layout): ...
    def showFindPopup(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.InputWidgets as InputWidgets
import QT4FormWidgets.PaintingUtils as PaintingUtils
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QT4FormWidgets.BaseValueFormWidget import BaseValueFormWidget as BaseValueFormWidget
from QT4FormWidgets.MultiStateBadge import MultiStateBadge as MultiStateBadge, ToggleStateBadge as ToggleStateBadge, ToggleValuePolicyState as ToggleValuePolicyState
from typing import Set, Tuple

class StringFormWidget(BaseValueFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _StringFormWidget__on_control_textChanged(self, text): ...
    def _StringFormWidget__on_defaultButton_clicked(self): ...
    def _buildControlWidget(self, layout): ...
    def _checkControlWidget(self): ...
    def _filterValueText(self, text): ...
    def _lockChanged(self, state): ...
    def _updateControlWidget(self): ...
    def selectAll(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Widgets.SceneGraphView.Filtering.Criteria
import UI4.Widgets.SceneGraphView.Filtering.Evaluator
import UI4.Widgets.SceneGraphView.Filtering.Target
import UI4.Widgets.SceneGraphView.SceneGraphHandle
from UI4.Widgets.SceneGraphView.SceneGraphLocationTranslation import GetParentPath as GetParentPath, IsLocationUnderTopLevelLocation as IsLocationUnderTopLevelLocation
from typing import Set, Tuple

class Rule:
    def __init__(self) -> None: ...
    def clearCriteria(self): ...
    def flushLocationMatchCache(self): ...
    def getComment(self) -> str | None: ...
    def getCriteria(self): ...
    def getDefaultCriteria(self) -> tuple[str, Criteria]: ...
    def getEvaluatorName(self) -> str: ...
    def getShowChildren(self): ...
    def getTarget(self) -> Target[UI4.Widgets.SceneGraphView.Filtering.Target.Target]: ...
    def isEnabled(self) -> bool: ...
    def isInverted(self) -> bool: ...
    def match(self, handle: SceneGraphHandle[UI4.Widgets.SceneGraphView.SceneGraphHandle.SceneGraphHandle], topLevelHandle: SceneGraphHandle[UI4.Widgets.SceneGraphView.SceneGraphHandle.SceneGraphHandle]) -> bool: ...
    def setComment(self, comment: str | None): ...
    def setCriteria(self, name: str, criteria: Criteria[UI4.Widgets.SceneGraphView.Filtering.Criteria.Criteria]): ...
    def setEnabled(self, enabled: bool): ...
    def setEvaluator(self, evaluator: Evaluator[UI4.Widgets.SceneGraphView.Filtering.Evaluator.Evaluator]): ...
    def setInverted(self, inverted: bool): ...
    def setShowChildren(self, showChildren): ...
    def setTarget(self, target: Target[UI4.Widgets.SceneGraphView.Filtering.Target.Target]): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import CacheManager as CacheManager
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import KatanaResources as KatanaResources
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import PyFnAttribute
import PyFnGeolib
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.ViewportWidget
import UI4.Widgets.ViewportWidget
import UI4.Util as Util
import Utils as Utils
import ViewerAPI as ViewerAPI
import _weakrefset
import typing
from Callbacks.Callbacks import Callbacks as Callbacks
from Nodes3DAPI.PortOpClient import PortOpClient
from PyUtilModule.VirtualKatana import FaceSelectionManager as FaceSelectionManager, LiveRenderAPI as LiveRenderAPI, RenderGlobals as RenderGlobals, RenderManager as RenderManager, ScenegraphManager as ScenegraphManager, WorkingSetManager as WorkingSetManager
from PyUtilModule.WorkingSet import WorkingSet as WorkingSet
from PyUtilModule.WorkingSetManager import WSM as WSM
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from UI4.Widgets.ViewportWidget import ViewportWidget as ViewportWidget
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class BaseViewerTab(BaseTab):
    ManipulationFreezePeriod: ClassVar[int] = ...
    UpdateTimer: ClassVar[None] = ...
    UpdateTimerInterval: ClassVar[float] = ...
    _BaseViewerTab__instances: ClassVar[_weakrefset.WeakSet] = ...
    viewOpChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent: PyQt5.QtWidgets.QWidget | None, flags: int = ...) -> None: ...
    def _BaseViewerTab__activeManipulatorsChangedCallback(self, attrs): ...
    def _BaseViewerTab__cleanup(self): ...
    def _BaseViewerTab__configureWorkingSets(self, viewerDelegate): ...
    def _BaseViewerTab__getSelectedLocationsCallback(self, emptyMessage): ...
    def _BaseViewerTab__getViewerDelegates(self, onlyFollowingViewNode: bool = ...): ...
    def _BaseViewerTab__getViewerProxyLoaderOp(self, proxyAttr: PyFnAttribute.GroupAttribute) -> PyFnAttribute.GroupAttribute: ...
    def _BaseViewerTab__onExpansionStateChanged(self, locationStateChanges: list[tuple], workingSet: WorkingSet, sender: object): ...
    def _BaseViewerTab__onPinnedStateChanged(self, locationStateChanges: list[tuple], workingSet: WorkingSet, sender: object): ...
    def _BaseViewerTab__onTabCreated(self, objectHash: int | None, tab: PyQt5.QtWidgets.QWidget): ...
    def _BaseViewerTab__onVisibilityStateChanged(self, _locationStateChanges, _workingSet, _sender): ...
    @staticmethod
    def _BaseViewerTab__on_destroyed(tab): ...
    def _BaseViewerTab__registerDestructionCallback(self): ...
    def _BaseViewerTab__selectFacesCallback(self, selectionAttr: PyFnAttribute.GroupAttribute): ...
    def _BaseViewerTab__selectLocationsCallback(self, locationsAttr): ...
    def _BaseViewerTab__setCursorPosition(self, groupAttr: PyFnAttribute.GroupAttribute, viewerDelegate: ViewerAPI.ViewerDelegate): ...
    def _BaseViewerTab__setCursorVisibility(self, visibleAttr: PyFnAttribute.IntAttribute): ...
    def _BaseViewerTab__setUpdateTimerConnectionEnabled(self, enabled: bool): ...
    def _BaseViewerTab__updateFromProjectSettings(self): ...
    def _applyViewerPluginExtensionTerminalOps(self, txn: PyFnGeolib.GeolibRuntimeTransaction, op: PyFnGeolib.GeolibRuntimeOp | None, viewerDelegate: ViewerAPI.ViewerDelegate) -> PyFnGeolib.GeolibRuntimeOp: ...
    @staticmethod
    def _checkGLVersion(requiredMajorVersion: int, requiredMinorVersion: int): ...
    def _cleanup(self): ...
    def _onUpdateEvent(self): ...
    def _on_cacheManager_flush(self, _eventType: str | None, _eventID: object, isNodeGraphLoading: bool = ...): ...
    def _on_implicitResolversToggled(self, eventType: str | None, eventID: object, implicitResolversActive: bool): ...
    def _on_lookThroughLocation_changeRequested(self, viewportName: str, location: str): ...
    def _on_lookThroughLocation_changed(self, viewportName: str, location: str, attrs: PyFnAttribute.GroupAttribute): ...
    def _on_lookThroughLocation_cooked(self, viewportName: str, location: str, attrs: PyFnAttribute.GroupAttribute): ...
    def _on_lookThroughLocation_doesNotExist(self, viewportName: str, location: str): ...
    def _on_node_setLocked(self, eventType: str | None, eventID: object, node: NodegraphAPI.Node, locked: bool): ...
    def _on_nodegraph_setCurrentTime(self, _eventType: str | None, _eventID: object, currentTime: float): ...
    def _on_nodegraph_setRootNode(self): ...
    def _on_parameter_setValue(self, args: seq): ...
    def _on_scenegraphManager_selectionChanged(self, eventType: str | None, eventID: object, selectedLocations, deselectedLocations, sender: object | None): ...
    def _on_selectedEnabledLocations_changed(self, delegate: ViewerAPI.ViewerDelegate): ...
    def _on_selectedLocations_cooked(self, delegate: ViewerAPI.ViewerDelegate): ...
    def _on_viewer_liveRenderFromViewerCameraChanged(self, eventType: str, eventID: object, followViewportCamera: bool, viewportWidget: ViewportWidget): ...
    def _on_viewer_visibilityFollowsWorkingSetChanged(self, _eventType: str | None, _eventID: object, visibilityFollowsWorkingSet: bool): ...
    def _on_viewportView_frozenStateChanged(self, viewportName: str, isViewFrozen: bool): ...
    def _resolveViewportWidget(self, viewport: Tuple[int, str, ViewportWidget | ViewerAPI.Viewport], viewerDelegate: ViewerAPI.ViewerDelegate) -> ViewportWidget: ...
    def _unregisterWorkingSetCallbacks(self): ...
    def _updateHook(self): ...
    def aboutToQuit(self): ...
    def activeManipulatorsChanged(self, viewportName: str, manipulatorName: str): ...
    def addViewerDelegate(self, delegateType: str, followsViewNode: bool = ...) -> ViewerAPI.ViewerDelegate: ...
    def addViewport(self, viewportType: str, viewportName: str, viewerDelegate: ViewerAPI.ViewerDelegate, postInitCallback: typing.Optional[typing.Callable] = ..., layers: Incomplete | None = ...) -> UI4.Widgets.ViewportWidget: ...
    def applyTerminalOps(self, txn: PyFnGeolib.GeolibRuntimeTransaction, op: PyFnGeolib.GeolibRuntimeOp | None, viewerDelegate: ViewerAPI.ViewerDelegate) -> PyFnGeolib.GeolibRuntimeOp: ...
    def closeEvent(self, event: PyQt5.QtGui.QCloseEvent): ...
    def filterManipulator(self, manipulatorName: str, selectedLocations: list, viewerDelegate: ViewerDelegate) -> bool: ...
    def flushCaches(self): ...
    @classmethod
    def flushInstanceCaches(cls): ...
    def freeze(self): ...
    def getNumberOfViewerDelegates(self) -> int: ...
    def getSelectedLocations(self) -> list[str]: ...
    def getViewOp(self, viewerDelegate): ...
    def getViewerDelegateByIndex(self, index: int) -> ViewerDelegate: ...
    def getViewportWidget(self, viewport: Tuple[int, str | ViewerAPI.Viewport], viewerDelegate: ViewerAPI.ViewerDelegate) -> UI4.Widgets.ViewportWidget.ViewportWidget | None: ...
    def getViewportWidgetByIndex(self, viewportIndex: int, viewerDelegate: ViewerAPI.ViewerDelegate) -> UI4.Widgets.ViewportWidget.ViewportWidget | None: ...
    def getViewportWidgetByName(self, viewportName: str, viewerDelegate: ViewerAPI.ViewerDelegate) -> UI4.Widgets.ViewportWidget.ViewportWidget | None: ...
    def getViewports(self, viewerDelegate: ViewerAPI.ViewerDelegate) -> typing.Iterator[Viewport]: ...
    def hideEvent(self, event: PyQt5.QtGui.QHideEvent): ...
    def installManipulatorFilter(self, filter_): ...
    def isViewerDelegateFollowingViewNode(self, index: int) -> bool: ...
    def loadViewerPluginExtensions(self): ...
    def removeManipulatorFilter(self, filter_: object): ...
    def removeViewport(self, viewportName, viewerDelegate: ViewerAPI.ViewerDelegate): ...
    def removeViewportByIndex(self, viewportIndex: int, viewerDelegate: ViewerAPI.ViewerDelegate): ...
    def setActiveLiveRenderCamera(self, useSameAsViewer: bool, path: str = ..., viewportCamera: Incomplete | None = ...): ...
    def setCamera(self, viewport: Tuple[int, str, ViewportWidget | ViewerAPI.Viewport], nameOrPath: str): ...
    def setDelegateViewOp(self, txn: PyFnGeolib.GeolibRuntimeTransaction, op: PyFnGeolib.GeolibRuntimeOp | None, viewerDelegate: ViewerAPI.ViewerDelegate) -> PyFnGeolib.GeolibRuntimeOp: ...
    def setEventHandlersEnabled(self, enabled: bool, isFreezeOrThaw: bool = ...): ...
    @classmethod
    def setUpdateTimerInterval(cls, interval: float): ...
    def setViewedNode(self, viewedNode: NodegraphAPI.Node | None, viewerDelegate: ViewerAPI.ViewerDelegate) -> PyFnGeolib.GeolibRuntimeOp: ...
    def setViewportWidgetUpdatesEnabled(self, viewportWidget: ViewportWidget, enabled: bool): ...
    def showEvent(self, event: PyQt5.QtGui.QShowEvent): ...
    def thaw(self): ...
    def updateLiveRenderCamera(self, location, camera): ...

class _ViewerDelegatePortOpClient(PortOpClient):
    _abc_impl: ClassVar[_abc_data] = ...
    __abstractmethods__: ClassVar[frozenset] = ...
    def __init__(self, viewerDelegate, port: NodegraphAPI.Port, updateOpCallback: typing.Callable) -> None: ...
    def modifyPostTraversalGraphState(self, graphState: NodegraphAPI.GraphState): ...
    def opChanged(self, op, graphState: NodegraphAPI.GraphState, txn): ...
    @property
    def op(self): ...
    @property
    def systemArgs(self): ...

def _FlushBaseViewerTabCaches(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from _typeshed import Incomplete
from typing import Set, Tuple

SMALL: float

class UnDegenerator:
    def __init__(self) -> None: ...
    def RGBAFromComponent(self, componentValue, component, baseColor: Incomplete | None = ..., lockedComponents: Incomplete | None = ...): ...
    def getNonDegenerateColor(self, component): ...
    def setColor(self, color): ...

def IsComponentDegenerate(componentValue, component): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
import KatanaResources as KatanaResources
import Nodes3DAPI as Nodes3DAPI
import PyQt5.QtWidgets
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import Nodes3DAPI.ScenegraphManager as ScenegraphManager
import UI4 as UI4
import Utils as Utils
from UI4.Widgets.ToolbarButton import ToolbarButton as ToolbarButton
from typing import Set, Tuple

class ImplicitFilterToggle(ToolbarButton):
    def __init__(self, parent) -> None: ...
    def _ImplicitFilterToggle__clicked(self): ...
    def _ImplicitFilterToggle__updateCallback(self, eventType, scenegraphID, implicitResolversActive, **kwargs): ...

class ImplicitFilterWarningLabel(PyQt5.QtWidgets.QFrame):
    def __init__(self, *args) -> None: ...
    def _ImplicitFilterWarningLabel__highlightedText(self, text): ...
    def _ImplicitFilterWarningLabel__implicitResolverListText(self, stage): ...
    def _ImplicitFilterWarningLabel__update(self, eventType, scenegraphID, implicitResolversActive, **kwargs): ...
    def enterEvent(self, ev): ...
    def leaveEvent(self, ev): ...
    def mousePressEvent(self, ev): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI.NodeExtensions as NodeExtensions
import NodegraphAPI as NodegraphAPI
import NodegraphAPI.NodegraphGlobals as NodegraphGlobals
import NodegraphAPI.Util as Util
import Utils as Utils
import typing
from _typeshed import Incomplete
from typing import Set, Tuple

AllChangeEventTypes: set
EventType_nodegraph_changed: str
EventType_nodegraph_changed_affects2DRender: str
NodeActivationChangeEventTypes: set
NodeGraphLoadBeginEventTypes: set
NodeGraphLoadEndEventTypes: set
NodePortChangeEventTypes: set
ParameterChangeEventTypes: set
PortConnectionChangeEventTypes: set
__filter_nodegraph_changed_affects2DRender_contributingNodes: None
__filter_nodegraph_changed_affects2DRender_isEnabled: bool
__filter_nodegraph_changed_affects2DRender_isLoading: bool
__filter_nodegraph_changed_affects2DRender_isThrottled: bool

def DoesNodeEffectViewNodes(node: NodegraphAPI.Node) -> bool: ...
def __filter_nodegraph_changed(eventType, eventID, **kwargs): ...
def __filter_nodegraph_changed_affects2DRender(eventType, eventID, **kwargs): ...
def __on_event_idle(eventType, eventID, **kwargs): ...
def __on_node_setViewed(eventType, eventID, node: typing.Optional[NodegraphAPI.Node] = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Nodes3DAPI_cmodule as Nodes3DAPI_cmodule
from . import AbstractTransform as AbstractTransform, Alembic_In as Alembic_In, AttributeCopy as AttributeCopy, AttributeEditor as AttributeEditor, AttributeFile_In as AttributeFile_In, AttributeSet as AttributeSet, BlockerCreate as BlockerCreate, BoundsAdjust as BoundsAdjust, CameraClippingPlaneEdit as CameraClippingPlaneEdit, CameraCreate as CameraCreate, CameraImagePlaneCreate as CameraImagePlaneCreate, CollectionCreate as CollectionCreate, ConstraintCache as ConstraintCache, ConstraintListEdit as ConstraintListEdit, Constraints as Constraints, CoordinateSystemDefine as CoordinateSystemDefine, CreateUtil as CreateUtil, DynamicParameterUtil as DynamicParameterUtil, ErrorNode as ErrorNode, Expressions as Expressions, FaceSelect as FaceSelect, FaceSelectionManager as FaceSelectionManager, Fork3D as Fork3D, GenericAssign as GenericAssign, GenericAssignRegistry as GenericAssignRegistry, GenericGeo as GenericGeo, GenericOp as GenericOp, GroupMerge as GroupMerge, HierarchyCopy as HierarchyCopy, HierarchyCreate as HierarchyCreate, IncomingSceneOpDelegates as IncomingSceneOpDelegates, IncomingTest as IncomingTest, InfoCreate as InfoCreate, InputGraphBasedCacheManager as InputGraphBasedCacheManager, Isolate as Isolate, LODMerge as LODMerge, LODRangeAssign as LODRangeAssign, LODSelect as LODSelect, LightCreate as LightCreate, LightLink as LightLink, LightLinkConstants as LightLinkConstants, LightLinkEdit as LightLinkEdit, LightLinkResolve as LightLinkResolve, LightLinkSetup as LightLinkSetup, LightListEdit as LightListEdit, LocationCreate as LocationCreate, LocationGenerate as LocationGenerate, LookFileBake as LookFileBake, LookFileBaking as LookFileBaking, LookFileGlobalsAssignBaseType as LookFileGlobalsAssignBaseType, LookFileMaterialsIn as LookFileMaterialsIn, LookFileMaterialsOut as LookFileMaterialsOut, LookFileOverrideEnable as LookFileOverrideEnable, LookFileResolve as LookFileResolve, Manifest as Manifest, Material as Material, MaterialResolve as MaterialResolve, MaterialStack as MaterialStack, Merge as Merge, NetworkMaterial as NetworkMaterial, NetworkMaterialInterfaceControls as NetworkMaterialInterfaceControls, NetworkMaterialParameterEdit as NetworkMaterialParameterEdit, NetworkMaterialSplice as NetworkMaterialSplice, Node3DEventTypes as Node3DEventTypes, Node3D_geolib3 as Node3D_geolib3, NodeLayoutAttributes as NodeLayoutAttributes, OpCacheManager as OpCacheManager, OpResolve as OpResolve, OpScript as OpScript, OpWrite as OpWrite, OutputChannelDefine as OutputChannelDefine, PonyCreate as PonyCreate, PortOpClient as PortOpClient, PrimitiveCreate as PrimitiveCreate, ProxyGeoPromote as ProxyGeoPromote, Prune as Prune, Rename as Rename, RenderNodeUtil as RenderNodeUtil, RenderOutputDefine as RenderOutputDefine, RenderSettingsBaseType as RenderSettingsBaseType, RendererProceduralArgs as RendererProceduralArgs, Rendering as Rendering, RenderingUtil as RenderingUtil, Resolve as Resolve, Resources as Resources, ReverseNormals as ReverseNormals, ScenegraphManager as ScenegraphManager, ScenegraphMask as ScenegraphMask, ShadingNodeArrayConnector as ShadingNodeArrayConnector, ShadingNodeBase as ShadingNodeBase, ShadingNodeSubnet as ShadingNodeSubnet, ShadingNodeUtil as ShadingNodeUtil, TerminalOpDelegates as TerminalOpDelegates, TimingUtils as TimingUtils, Transform3D as Transform3D, TransformEdit as TransformEdit, TransformUtil as TransformUtil, Unfork3D as Unfork3D, UpdateModes as UpdateModes, VariableSwitchUtil as VariableSwitchUtil, VelocityApply as VelocityApply, WorkingSetDebug as WorkingSetDebug, ZoomToRect as ZoomToRect
from .TerminalOpDelegates import TerminalOpDelegate as TerminalOpDelegate
from Nodes3DAPI.IncomingSceneOpDelegates import IncomingSceneOpDelegate as IncomingSceneOpDelegate, OutgoingAttributesDelegate as OutgoingAttributesDelegate, RegisterIncomingSceneOpDelegate as RegisterIncomingSceneOpDelegate, RegisterOutgoingAttributesDelegate as RegisterOutgoingAttributesDelegate
from Nodes3DAPI.Node3D import ApplyImplicitResolverOps as ApplyImplicitResolverOps, ApplyOp as ApplyOp, ApplyRenderSettingsToGraphState as ApplyRenderSettingsToGraphState, CreateClient as CreateClient, Get3DPortFromNode as Get3DPortFromNode, Get3DSourceFromNodeInput as Get3DSourceFromNodeInput, GetExtraParameterValueSourceNodePorts as GetExtraParameterValueSourceNodePorts, GetExtraParameterValueSourceNodes as GetExtraParameterValueSourceNodes, GetGeometryProducer as GetGeometryProducer, GetOp as GetOp, GetOpChain as GetOpChain, GetRegisteredImplicitResolvers as GetRegisteredImplicitResolvers, GetRegisteredTerminalOpDelegate as GetRegisteredTerminalOpDelegate, GetRegisteredTerminalOpDelegateNames as GetRegisteredTerminalOpDelegateNames, GetRenderOp as GetRenderOp, GetRenderProducer as GetRenderProducer, GetRenderTerminalOpSpecs as GetRenderTerminalOpSpecs, GetRenderTerminalOps as GetRenderTerminalOps, GetRenderThreads as GetRenderThreads, ImplicitResolverStage as ImplicitResolverStage, Node3D as Node3D, RegisterImplicitResolver as RegisterImplicitResolver, RegisterTerminalOpDelegate as RegisterTerminalOpDelegate, SetExtraParameterValueSourceNodes as SetExtraParameterValueSourceNodes, SetRenderThreads as SetRenderThreads, UnregisterTerminalOpDelegate as UnregisterTerminalOpDelegate
from Nodes3DAPI.Node3D_geolib3 import ActivatePort as ActivatePort, CommitChanges as CommitChanges, DeactivatePort as DeactivatePort, GetDefaultSourceOp as GetDefaultSourceOp, GetRuntime as GetRuntime, IsProcessing as IsProcessing, ManualUpdate as ManualUpdate, MarkPortOpClientDirty as MarkPortOpClientDirty, RegisterCommitIdCallback as RegisterCommitIdCallback, RegisterPortOpClient as RegisterPortOpClient, UnregisterPortOpClient as UnregisterPortOpClient, UpdatePortOpClients as UpdatePortOpClients
from Nodes3DAPI.NodeTypeBuilder import NodeTypeBuilder as NodeTypeBuilder
from Nodes3DAPI.RenderNode import RenderNode as RenderNode
from Nodes3DAPI.Rendering import CancelAllRenders as CancelAllRenders, CancelRender as CancelRender, GetRenderCommandLine as GetRenderCommandLine, RenderNode3D as RenderNode3D, SignalRender as SignalRender
from Nodes3DAPI_cmodule import BuildAttrListFromDynamicParameterGroup as BuildAttrListFromDynamicParameterGroup, DefaultDAPCookOrder as DefaultDAPCookOrder, EvaluateBoolExpresion as EvaluateBoolExpresion
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from _typeshed import Incomplete
from typing import Set, Tuple

def GetConnectionNames(sourceNodeName: str, sourcePortName: str, isArray: bool) -> tuple[str, str]: ...
def IsArrayConnection(connectedNodeName: str) -> bool: ...
def ReportError(errorMessage: str, errorMessages: Incomplete | None = ..., loggingFunction: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from typing import ClassVar, Set, Tuple

class VariablesPopup(PyQt5.QtWidgets.QFrame):
    hideSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    showSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self) -> None: ...
    def hideEvent(self, e): ...
    def popup(self, globalPos): ...
    def showEvent(self, e): ...

class VariablesPopupButton(PyQt5.QtWidgets.QFrame):
    clicked: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent) -> None: ...
    def _VariablesPopupButton__clicked(self): ...
    def _VariablesPopupButton__on_updateTextTimer_timeout(self): ...
    def _VariablesPopupButton__popupHidden(self): ...
    def _VariablesPopupButton__popupShow(self): ...
    def _VariablesPopupButton__possibleChange(self, args): ...
    def _VariablesPopupButton__updateText(self): ...
    def mousePressEvent(self, ev): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import Utils as Utils
from _typeshed import Incomplete
from typing import Set, Tuple

class GetSetAction(PyQt5.QtWidgets.QAction):
    def __init__(self, name, parent, value, setFunc, getFunc, expandTuple: bool = ..., shortcut: Incomplete | None = ..., isBoolean: bool = ..., useWeakMethod: bool = ...) -> None: ...
    def doGet(self): ...
    def doSet(self): ...

class GetSetMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, name, parent) -> None: ...
    def _GetSetMenu__doAboutToShow(self): ...
    def _GetSetMenu__doTriggered(self, action): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from NodegraphAPI.ParameterValueOption import ExistingLocations as ExistingLocations, Immediate as Immediate, LookFileResolve as LookFileResolve, MaterialResolve as MaterialResolve, NewLocation as NewLocation, OpResolve as OpResolve, ParameterValueOption as ParameterValueOption, Primary as Primary
from typing import Set, Tuple

AllOptions: list
